@@ -119,7 +119,7 @@ class Philly(BackendBase):
                     name = name[:index]
                 job["run_name"] = name
 
-            lb = ReportBuilder(self.config, self.store, client=None)
+            lb = ReportBuilder(self.config, self.store)
             text, rows = lb.build_formatted_table(jobs, columns)
             console.print(text)
 
