@@ -10,14 +10,14 @@ here, check to see if you need to also change it in psm.py.
 '''
 
 #Sync the BUILD string changes with: setup.py, CHANGELOG.md
-BUILD = "version: 0.0.223, build: Sep-05-2020"
+BUILD = "version: 0.0.227, build: Sep-17-2020"
     
 INFO_CONTAINER_V1 = "xt-store-info"
 INFO_CONTAINER_V2 = "00-store-info"
+STORAGE_FORMAT_V2 = "2"
+STORAGE_FORMAT_V1 = "1"
 
 INFO_DIR = "__info__"
-STORAGE_FORMAT = "2"
-STORAGE_FORMAT_V1 = "1"
 STORAGE_INFO_FILE = INFO_DIR + "/storage_info.json"
 NODE_PREFIX = "_"
 
