@@ -1149,14 +1149,22 @@ class XTController(rpyc.Service):
         child_env["XT_MONGO_CONN_STR"] = utils.text_to_base64(mc)
 
         # update XT_OUTPUT_DIR and XT_OUTPUT_MNT for child run path
-        mnt_path = os.getenv("XT_OUTPUT_MNT")
+        output_path = os.getenv("XT_OUTPUT_DIR")
+        log_info("parent output path", output_path)
         parent_name = run_info.parent_name
-        if mnt_path and parent_name:
-            child_mnt_path = mnt_path.replace(parent_name, run_name)
-            log_info("child XT_OUTPUT_MNT", child_mnt_path)
+        log_info("parent name", parent_name)
 
-            child_env["XT_OUTPUT_MNT"] = child_mnt_path
-            child_env["XT_OUTPUT_DIR"] = child_mnt_path
+        if output_path and parent_name:
+            child_output_path = output_path.replace(parent_name, run_name)
+            log_info("child XT_OUTPUT_DIR", child_output_path)
+        else:
+            child_output_path = output_path
+
+        child_env["XT_OUTPUT_DIR"] = child_output_path
+        child_env["XT_OUTPUT_MNT"] = child_output_path
+
+        # ensure dir exists and is empty (local machine)
+        file_utils.ensure_dir_clean(child_output_path)
 
         pp = os.getenv("PYTHONPATH")
         log_info("PYTHONPATH", pp)
