@@ -25,7 +25,7 @@ def early_exit_without_error(msg=None):
     This is not an error; just information that request terminated early.
     '''
     if msg:
-        console.print(msg)
+        console.print(msg, flush=True)
     sys.exit(0)
 
 def syntax_error_exit():
@@ -42,7 +42,7 @@ def process_exception(ex_type, ex_value, ex_traceback, exit_app=True):
     if issubclass(ex_type, XTUserException):
         # user-related exception: don't show stack trace
         console.print()
-        console.print("Exception: " + msg)
+        console.print("Exception: " + msg, flush=True)
        
         if issubclass(ex_type, SyntaxError):
             # show syntax/args for command
@@ -61,7 +61,7 @@ def process_exception(ex_type, ex_value, ex_traceback, exit_app=True):
             raise ex_value
         else:
             # for use in REPL command: just show stack trace but do not exit
-            traceback.print_exception(ex_type, ex_value, ex_traceback)
+            traceback.print_exception(ex_type, ex_value, ex_traceback, flush=True)
 
     if exit_app:
         # use "os._exit" to prevent a 2nd exception from being thrown
@@ -132,7 +132,7 @@ def argument_error(arg_type, token):
     else:
         token2 = "--" + token
     #syntax_error("expected {}, but found '{}'.  Did you mean '{}'?".format(arg_type, token, token2))    
-    syntax_error("unrecognized argument: {}?".format(token))
+    syntax_error("unrecognized argument: {}".format(token))
 
 def warning(msg):
-    console.print("Warning: " + msg)
\ No newline at end of file
+    console.print("Warning: " + msg, flush=True)
\ No newline at end of file
