@@ -167,7 +167,7 @@ class ImplStorageApi():
         job["ws_name"] = new_workspace
 
         # add to mongo
-        self.store.mongo.update_job(job)
+        self.store.mongo.create_mongo_job(job)
 
     def import_job_storage_blobs(self, storage_job_path, new_workspace, prev_job_id, new_job_id):
 
@@ -554,6 +554,8 @@ class ImplStorageApi():
     def create_file_accessor(self, use_blobs, share, ws_name, exper_name, job_name, run_name):
         # use --experiment option only here
         if share:
+            self.store.init_store_utils_if_needed()
+
             ws_name = store_utils.make_share_name(share)
             fs = self.store.root_files(ws_name, use_blobs=use_blobs)
         elif run_name:
@@ -602,7 +604,7 @@ class ImplStorageApi():
             for record in records:
                 console.print(record)
         else:
-            builder = ReportBuilder(self.config, self.store, client)
+            builder = ReportBuilder(self.config, self.store)
 
             args = {}
             args["columns"] = ["time", "event", "data"]
