@@ -24,10 +24,12 @@ MAX = "_MAX_"
 class PlotBuilder():
     def __init__(self, run_names, col_names, x_col, layout, break_on, title, show_legend, plot_titles,
             legend_titles, smoothing_factor, plot_type, timeout,
-            aggregate, shadow_type, shadow_alpha, run_log_records, style, show_toolbar, max_runs, max_traces,
+            aggregate, shadow_type, shadow_alpha, run_log_records, style, show_toolbar, max_traces,
             group_by, error_bars, show_plot, save_to, x_label, colors, color_map, color_steps, legend_args, 
-            x_min, x_max, y_min, y_max, alpha, line_size, timebase, x_format, y_format, plot_args):
+            x_min, x_max, y_min, y_max, alpha, line_size, timebase, x_format, y_format, 
+            x_share, y_share, background_color, grid_color, clean, width, height, plot_args):
         
+        self.group_names = None
         self.run_names = run_names
         self.col_names = col_names
         self.x_col = x_col
@@ -46,9 +48,8 @@ class PlotBuilder():
         self.run_log_records = run_log_records
         self.style = utils.zap_none(style)
         self.show_toolbar = show_toolbar
-        self.max_runs = max_runs
         self.max_traces = max_traces
-        self.group_by = group_by if group_by else "run"
+        self.group_by = group_by if group_by and group_by != "none" else "run"
         self.error_bars = utils.zap_none(error_bars)
         self.show_plot = show_plot
         self.save_to = save_to
@@ -64,6 +65,13 @@ class PlotBuilder():
         self.timebase = timebase
         self.x_format = x_format
         self.y_format = y_format
+        self.x_share = x_share
+        self.y_share = y_share
+        self.background_color = background_color
+        self.grid_color = grid_color
+        self.clean = clean
+        self.width = width 
+        self.height = height
 
         if colors:
             self.colors = colors
@@ -90,7 +98,9 @@ class PlotBuilder():
 
     def build(self):
 
-        data_frames_by_cols = self.build_data_frames()
+        group_order_dict = {}
+
+        data_frames_by_cols = self.build_data_frames(group_order_dict)
         if data_frames_by_cols:
 
             for cols, dfx in data_frames_by_cols.items():
@@ -101,7 +111,25 @@ class PlotBuilder():
             if self.show_plot or self.save_to:
                 self.plot_data(data_frames_by_cols)
 
-    def build_data_frames(self):
+    def get_group_value(self, group_by, run, node, job, experiment, workspace):
+        value = None
+
+        if group_by == "run":
+            value = run
+        elif group_by == "node":
+            value = node
+        elif group_by == "job":
+            value = job
+        elif group_by == "experiment":
+            value = experiment
+        elif group_by == "workspace":
+            value = experiment
+        else:
+            raise Exception("unrecognized group-by name: " + str(group_by))
+
+        return value
+
+    def build_data_frames(self, group_order_dict):
         '''
         1. for each run, collect the reported metrics as metric sets (by reported col list)
 
@@ -118,10 +146,17 @@ class PlotBuilder():
         for i, record in enumerate(self.run_log_records):
             # extract metrics for this run
             run = record["run_name"]
+            console.diag(run)
+
             node = utils.node_id(record["node_index"])
             job = record["job_id"]
             experiment = record["exper_name"]
             workspace = record["ws_name"]
+
+            group_value = self.get_group_value(self.group_by, run, node, job, experiment, workspace)
+            if not group_value in group_order_dict:
+                group_order_dict[group_value] = 1
+
             search_style = utils.safe_value(record, "search_style")
             if search_style and search_style != "single":
                 # parent run with children - skip it
@@ -129,15 +164,11 @@ class PlotBuilder():
 
             log_records = record["log_records"]
 
-            metric_sets = run_helper.build_metrics_sets(log_records, timebase=self.timebase)
+            metric_sets = run_helper.build_metrics_sets(log_records, timebase=self.timebase, cleanup=self.clean)
             if not metric_sets:
                 no_metrics.append(run)
                 #continue    # SKIP this run
 
-            if self.max_runs and len(pp_run_names) >= self.max_runs:
-                used_max = True
-                break
-
             if self.x_col:
                 # ensure x_col is in this run
                 if not self.is_col_in_sets(metric_sets, self.x_col):
@@ -189,11 +220,13 @@ class PlotBuilder():
                     pass
 
                 # add run_name column
-                df["run"] = [run] * df.shape[0]
-                df["node"] = [node] * df.shape[0]
-                df["job"] = [job] * df.shape[0]
-                df["experiment"] = [experiment] * df.shape[0]
-                df["workspace"] = [workspace] * df.shape[0]
+                count = df.shape[0]
+
+                df["run"] = [run] * count
+                df["node"] = [node] * count
+                df["job"] = [job] * count
+                df["experiment"] = [experiment] * count
+                df["workspace"] = [workspace] * count
 
                 if not cols in data_frames_by_cols:
                     data_frames_by_cols[cols] = df
@@ -207,8 +240,8 @@ class PlotBuilder():
         if no_metrics:
             console.print("\nnote: following runs currently have no logged metrics: \n    {}\n".format(", ".join(no_metrics)))
 
-        if used_max:
-            console.print("plotting first {} runs (use --max-runs to override)".format(self.max_runs))
+        # if used_max:
+        #     console.print("plotting first {} runs (use --max-runs to override)".format(self.max_runs))
         elif pp_run_names:
             console.print("plotting {} runs...".format(len(pp_run_names)))
         else:
@@ -227,6 +260,9 @@ class PlotBuilder():
         # update our list of run_names to proces
         self.run_names = pp_run_names
 
+        # update our list of group names (the order they will be processed in)
+        self.group_names = list(group_order_dict)
+
         return data_frames_by_cols
 
     def get_agg_df(self, df, agg_op, df_cols):
@@ -380,8 +416,71 @@ class PlotBuilder():
                         stat_values = df[ stat_col ].to_numpy(dtype=float)
                     break
 
+        if x_values is not None and y_values is not None:
+            if self.has_line_segments(x_values):
+
+                x_values, y_values, stat_values = \
+                    self.join_line_segments(x_values, y_values, stat_values)
+
         return x_values, y_values, stat_values
 
+    def has_line_segments(self, x_values):
+        last_x = None
+        has_segments = False
+
+        for x in x_values:
+            if last_x and x <= last_x:
+                has_segments = True
+                break
+            last_x = x
+
+        return has_segments
+
+    def join_line_segments(self, x_values, y_values, stat_values):
+        '''
+        Processing:
+            Converts x and stat values into numpy masked arrays, which are honored
+            by matplotlib to correctly draw line segments.  For every line segment
+            detected, we insert a dummy point to be ignored in the mask.
+        '''
+        last_x = None
+        has_segments = False
+        new_x = []
+        new_y = []
+        new_stat = []
+        mask = []
+
+        for i, x in enumerate(x_values):
+            if last_x and x <= last_x:
+                # insert a dummy point to ignore
+                new_x.append(x)
+                new_y.append(0)
+                if stat_values:
+                    new_stat.append(0)
+                mask.append(True)
+
+            # add current point
+            new_x.append(x)
+            new_y.append(y_values[i])
+            if stat_values:
+                new_stat.append(stat_values[i])
+            mask.append(False)
+            last_x = x
+
+        new_x = np.array(new_x)
+        new_y = np.array(new_y)
+        new_stat = np.array(new_stat) if stat_values else stat_values
+
+        # build mask array for y
+        import numpy.ma as ma
+        new_y = ma.array(new_y, mask=mask)
+
+        # build mask array for new_stat
+        if new_stat:
+            new_stat = ma.array(new_stat, mask=mask)
+
+        return new_x, new_y, new_stat
+
     def plot_data(self, data_frames_by_cols):
         console.diag("starting to plot data")
 
@@ -400,20 +499,25 @@ class PlotBuilder():
             sns.set_style(self.style)
 
         # gather group names (usually all are in the first dataset, but not always)
-        # preserve the order names were found in (don't use a set)
-        group_names = {}     # python 3.6+ preserves name order in dicts
+        # # preserve the order names were found in (don't use a set)
+        # group_names = {}     # python 3.6+ preserves name order in dicts
 
-        for dfx in data_frames_by_cols.values():
-            name_list = dfx[self.group_by].unique()
+        # for dfx in data_frames_by_cols.values():
+        #     if not dfx.empty:
 
-            for name in name_list:
-                if not name in group_names:
-                    group_names[name] = 1
+        #         # get list of unique group_by values in dfx
+        #         #name_list = dfx[self.group_by].order_by("user_run_index").unique()
+        #         name_list = dfx[self.group_by]
+
+        #         name_list = name_list.unique()
+
+        #         for name in name_list:
+        #             if not name in group_names:
+        #                 group_names[name] = 1
         
-        group_names = list(group_names)
+        # group_names = list(group_names)
 
-        # this will sort the group names in a number-smart way 
-        #group_names.sort(key=utils.natural_keys)
+        group_names = self.group_names
 
         group_count = len(group_names)
         col_count = len(self.col_names)
@@ -458,12 +562,11 @@ class PlotBuilder():
             self.legend_titles = [legend_text]
 
         # configure matplotlib for our subplots
-        sharex = True
-        sharey = True
+        sharex = self.x_share
+        sharey = self.y_share
 
         #plt.close()
-        window_size = (14, 6)
-        #window_size = (20, 12)
+        window_size = (self.width, self.height)
 
         fig, plots = plt.subplots(plot_rows, plot_cols, figsize=window_size, sharex=sharex, sharey=sharey, constrained_layout=True)
         if not isinstance(plots, np.ndarray):
@@ -608,6 +711,12 @@ class PlotBuilder():
 
     def plot_middle(self, data_frames_by_cols, ax, group_name, col, x_col, x_label, line_index, plot_index):
         
+        if self.background_color:
+            ax.set_facecolor((self.background_color))            
+
+        if self.grid_color:
+            ax.grid(color=self.grid_color)   # , linestyle='-', linewidth=2)
+
         color_index = line_index % len(self.colors)
         color = self.colors[color_index]
 
@@ -628,8 +737,13 @@ class PlotBuilder():
                 x, y_mean, _ = self.get_xy_values(data_frames_by_cols, group_name, self.x_col, col + MEAN, None)
                 x, y_stat, _ = self.get_xy_values(data_frames_by_cols, group_name, self.x_col, col + stat_name, None)
 
-                y = y_mean - y_stat
-                y2 = y_mean + y_stat
+                if x is None or y_mean is None:
+                    x = None
+                    y = None
+                    y2 = None
+                else:
+                    y = y_mean - y_stat
+                    y2 = y_mean + y_stat
 
             self.plot_inner(ax, group_name, col, self.x_col, x_label, line_index, plot_index, x_values=x, y_values=y, 
                 color=color, alpha=self.shadow_alpha, use_y_label=False, y2_values=y2)
@@ -661,66 +775,73 @@ class PlotBuilder():
         import seaborn as sns
         from matplotlib.ticker import MaxNLocator
 
-        if y_values is not None and len(y_values):
-            if x_values is None:        
-                x_values = range(len(y_values))
+        if y_values is None or len(y_values) == 0:  
+            # draw with dummy data to keep legend entries consistent and matching user specifed order
+            alpha = 0
+            if x_values:
+                y_values = [0] * len(x_values)
             else:
-                ax.set_xlabel(x_label)
+                y_values = [0]
 
-            if use_y_label:
-                line_title = self.legend_titles[line_index % len(self.legend_titles)]
-                line_title = self.fixup_text(line_title, group_name, col)
-            else:
-                line_title = None
-                
-            console.detail("x_values=", x_values)
-            console.detail("y_values=", y_values)
-            console.detail("y2_values=", y2_values)
+        if x_values is None:        
+            x_values = range(len(y_values))
+        else:
+            ax.set_xlabel(x_label)
 
-            num_y_ticks = 10
-            ax.get_yaxis().set_major_locator(MaxNLocator(num_y_ticks))
-            #color = self.colors[line_index % len(self.colors)]
-
-            cap_size = 5
-            is_range_plot = bool(y2_values is not None)
-
-            # our default attributes
-            kwargs = {"label": line_title, "color": color, "alpha": alpha, "linewidth": self.line_size}
+        if use_y_label:
+            line_title = self.legend_titles[line_index % len(self.legend_titles)]
+            line_title = self.fixup_text(line_title, group_name, col)
+        else:
+            line_title = None
             
-            if not is_range_plot:
-                kwargs["capsize"] = cap_size
+        console.detail("x_values=", x_values)
+        console.detail("y_values=", y_values)
+        console.detail("y2_values=", y2_values)
 
-            # let user override
-            if self.plot_args and not is_range_plot:
+        num_y_ticks = 10
+        ax.get_yaxis().set_major_locator(MaxNLocator(num_y_ticks))
+        #color = self.colors[line_index % len(self.colors)]
 
-                for name, value in self.plot_args.items():
-                    value = utils.make_numeric_if_possible(value)
-                    kwargs[name] = value
+        cap_size = 5
+        is_range_plot = bool(y2_values is not None)
 
-            #cmap = self.get_seaborn_color_map("muted")
-            if self.plot_type == "line":
+        # our default attributes
+        kwargs = {"label": line_title, "color": color, "alpha": alpha, "linewidth": self.line_size}
+        
+        if not is_range_plot:
+            kwargs["capsize"] = cap_size
 
-                if is_range_plot:
+        # let user override
+        if self.plot_args and not is_range_plot:
 
-                    # RANGE plot
-                    ax.fill_between(x_values, y_values, y2_values, **kwargs)
+            for name, value in self.plot_args.items():
+                value = utils.make_numeric_if_possible(value)
+                kwargs[name] = value
 
-                elif x_values is not None:
+        #cmap = self.get_seaborn_color_map("muted")
+        if self.plot_type == "line":
 
-                    # X/Y LINE plot
-                    trace = ax.errorbar(x_values, y_values, yerr=err_values, **kwargs)  
-                else:
+            if is_range_plot:
 
-                    # LINE plot
-                    ax.errorbar(y_values, '-', label=line_title, yerr=err_values, **kwargs)
+                # RANGE plot
+                ax.fill_between(x_values, y_values, y2_values, **kwargs)
 
+            elif x_values is not None:
+
+                # X/Y LINE plot
+                trace = ax.errorbar(x_values, y_values, yerr=err_values, **kwargs)  
             else:
-                # for now, we can get lots of milage out of line plot (errorbars, scatter, scatter+line)
-                # so keep things simple and just support 1 type well 
-                errors.syntax_error("unknown plot type={}".format(self.plot_type))
+
+                # LINE plot
+                ax.errorbar(y_values, '-', label=line_title, yerr=err_values, **kwargs)
+
+        else:
+            # for now, we can get lots of milage out of line plot (errorbars, scatter, scatter+line)
+            # so keep things simple and just support 1 type well 
+            errors.syntax_error("unknown plot type={}".format(self.plot_type))
 
         # this must be called AFTER plotting
-        line_title = self.add_plot_legend_and_title(ax, plot_index, line_index, group_name, col, use_y_label)
+        self.add_plot_legend_and_title(ax, plot_index, line_index, group_name, col, use_y_label)
 
     def fixup_text(self, text, group_name, col):
         text = text.replace("$job", group_name)
