@@ -17,7 +17,8 @@ from xtlib import process_utils
 from xtlib.helpers.xt_config import get_merged_config
 
 CONTROLLER_NAME_PATTERN = "xtlib.controller"
-PY_RUN_CONTROLLER ="__run_controller__.py"
+PY_RUN_CONTROLLER = "__run_controller__.py"
+PSM_NAME_PATTERN = "psm.py"
 
 class LocalPsmClient():
     def __init__(self):
@@ -25,6 +26,8 @@ class LocalPsmClient():
         self.psm_queue_path = os.path.expanduser(constants.PSM_QUEUE)
         self.psm_log_path = os.path.expanduser(constants.PSM_LOGDIR)
         self.cwd_path = os.path.expanduser(constants.CWD)
+        self.xt_path = os.path.expanduser(constants.XT_HOME)
+        self.controller_cwd_path = utils.get_controller_cwd(self.box_is_windows, is_local=True)
 
         # ensure our required dirs have been created
         if not os.path.exists(self.cwd_path):
@@ -33,15 +36,20 @@ class LocalPsmClient():
         if not os.path.exists(self.psm_queue_path):
             os.makedirs(self.psm_queue_path)
 
-    def enqueue(self, team, job, node, fn_zip):
+    def enqueue(self, team, ws_name, job, run, node_index, fn_zip):
         # copy file to box (with unique name)
         #guid = str(uuid.uuid4())
         ticks = time.time()
 
-        # copy SCRIPT
-        fn_entry = "{}.{}.{}.{}.zip".format(team, job, node, int(10*ticks))
-        fn_dest = os.path.join(self.psm_queue_path, fn_entry)
-        shutil.copyfile(fn_zip, fn_dest)
+        # copy ENTRY as .tmp
+        fn_tmp_entry = "{}.{}.{}.{}.{}.{}.tmp".format(team, ws_name, job, run, node_index, int(10*ticks))
+        fn_tmp_dest = os.path.join(self.psm_queue_path, fn_tmp_entry)
+        shutil.copyfile(fn_zip, fn_tmp_dest)
+
+        # now, rename the file to .zip so that PSM can see it
+        fn_entry = os.path.splitext(fn_tmp_entry)[0] + ".zip"
+        fn_dest = file_utils.path_join(self.psm_queue_path, fn_entry, for_windows=self.box_is_windows)
+        os.rename(fn_tmp_dest, fn_dest)
 
         return fn_entry
 
@@ -69,9 +77,12 @@ class LocalPsmClient():
 
         for p in processes:
             try:
-                if p.name().lower().startswith("python"):
-                    #print("process name: {}".format(p.name()))
+                name = p.name().lower()
+                #print("process name: {}".format(name))
+
+                if name.startswith("python"):
                     cmd_line = " ".join(p.cmdline())
+                    #print("cmd_line: {}".format(cmd_line))
 
                     if constants.PSM in cmd_line:
                         psm_count += 1
@@ -100,18 +111,102 @@ class LocalPsmClient():
             
         return controller_process
 
-    def start_psm_if_needed(self):
+    def _get_wrapper_process(self):
+        processes = psutil.process_iter()
+        wrapper_process = None
+
+        for p in processes:
+            try:
+                name = p.name().lower()
+                if name in ["bash", "cmd.exe"]:
+                    #print("process name: {}".format(p.name()))
+                    cmd_line = " ".join(p.cmdline())
+                    #print("cmd_line=", cmd_line)
+
+                    if constants.FN_WRAPPED_SH in cmd_line or constants.FN_WRAPPED_BAT in cmd_line:
+                        wrapper_process = p
+                        break
+
+            except BaseException as ex:
+                pass
+            
+        return wrapper_process
+
+    def _get_psm_process(self):
+        processes = psutil.process_iter()
+        psm_process = None
+
+        for p in processes:
+            try:
+                if p.name().lower().startswith("python"):
+                    #print("process name: {}".format(p.name()))
+                    cmd_line = " ".join(p.cmdline())
+
+                    if PSM_NAME_PATTERN in cmd_line:
+                        psm_process = p
+                        break
+
+            except BaseException as ex:
+                pass
+            
+        return psm_process
+
+    def _get_docker_container(self):
+        docker_count = 0
+        fn_image_name = os.path.join(self.controller_cwd_path, constants.FN_IMAGE_NAME)
+
+        if os.path.exists(fn_image_name):
+            # current run is a docker container run
+            with open(fn_image_name, "rt") as infile:
+                image_name = infile.read().strip()
+    
+            # count the number of entries we have running in docker
+            output = os.popen('docker ps').read()
+            containers = output.split("\n")[1:]
+
+            # filter to matches of our image name
+            containers = [con for con in containers if image_name in con]
+            docker_count = len(containers)
+
+        return docker_count
+
+    def restart_psm_if_needed(self):
+        '''
+        processing:
+            - if PSM is running on old psm.py, kill the process and restart it.  
+            - if PMS is not running, start it.
+        '''
+        kill_needed = False
+        start_needed = False
+
+        fn_src = os.path.join(file_utils.get_my_file_dir(__file__), constants.PSM)
+        fn_dest = os.path.join(self.xt_path, constants.PSM)
+
         running = self._is_psm_running()
         #print("PSM running=", running)
 
-        if not running:
+        if running:
+            # do file contents match?
+            text_src = file_utils.read_text_file(fn_src)
+            text_dest = file_utils.read_text_file(fn_dest) if os.path.exists(fn_dest) else None
+
+            if text_src != text_dest:
+                kill_needed = True
+        else:
+            start_needed = True
+
+        if kill_needed:
+            p = self._get_psm_process()
+            if p:
+                p.kill()
+            start_needed = True
+
+        if start_needed:
             # always copy psm.py (for xt dev/debug purposes)
-            fn_src = os.path.join(file_utils.get_my_file_dir(__file__), constants.PSM)
-            fn_dest = os.path.join(self.cwd_path, constants.PSM)
             shutil.copyfile(fn_src, fn_dest)
 
             # run psm
-            fn_log = os.path.join(self.cwd_path, constants.PSMLOG)
+            fn_log = os.path.join(self.xt_path, constants.PSMLOG)
 
             if self.box_is_windows:
                 cmd_parts = ["cmd", "/c", "python -u {} > {}".format(fn_dest, fn_log)]
@@ -119,16 +214,14 @@ class LocalPsmClient():
                 cmd_parts = ["bash", "-c", "python -u {} > {}".format(fn_dest, fn_log)]
             
             # put log in known dir
-            fn_run_log = os.path.expanduser("~/.xt/cwd/runpsm.log")
+            fn_run_log = os.path.expanduser("~/.xt/runpsm.log")
             
             process_utils.start_async_run_detached(cmd_parts, self.cwd_path, fn_run_log)
 
     def get_running_entry_name(self):
         text = None
 
-        controller_cwd = utils.get_controller_cwd(self.box_is_windows, is_local=True)
-
-        fn_current = os.path.join(controller_cwd, constants.CURRENT_RUNNING_ENTRY)
+        fn_current = os.path.join(self.controller_cwd_path, constants.CURRENT_RUNNING_ENTRY)
         if os.path.exists(fn_current):
             text = file_utils.read_text_file(fn_current).strip()
 
@@ -143,9 +236,15 @@ class LocalPsmClient():
         else:
             text = self.get_running_entry_name()
             if text == fn_entry:
-                # entry might be running; is the controller active?
+                # entry might be running; is the runner script OR controller OR docker container active?
                 if self._get_controller_process():
                     status = "running"
+                elif self._get_wrapper_process():
+                    status = "running"
+                elif self._get_docker_container():
+                    status = "running"
+                else:
+                    status = "completed"
 
         return status
 
