@@ -2,9 +2,10 @@
 # Copyright (c) Microsoft Corporation.
 # Licensed under the MIT license.
 #
-# qfe: a quick-front-end builder for XT 
+# qfe: a quick-front-end builder for XT (parses commands, arguments, and options & dispatches to associated code).
 import os
 import sys
+import copy
 import functools
 
 from xtlib.console import console
@@ -13,6 +14,7 @@ from xtlib.helpers.scanner import Scanner
 from xtlib import utils
 from xtlib import errors
 from xtlib import run_helper
+from xtlib import job_helper
 
 '''
 Definitions:
@@ -31,6 +33,10 @@ commands = {}               # nested dictionary, each level index by next comman
 commands_by_name = {}       # key = function name
 current_dispatcher = None
 
+# workaround for "commands" getting corrupted during quicktest
+save_commands = None
+dispatch_counter = 0
+
 # special cmd_info's and funcs
 root_cmd_info = None
 current_cmd_info = None         # for command function being defined as function decorators are processed
@@ -44,14 +50,14 @@ debug_decorators = False
 first_command = True
 
 def is_xt_object(name):
-   if run_helper.is_run_name(name):
-      match = True
-#    elif name.startswith("job") and name[3:].isdigit():
-#       match = True
-   else:
-      match = False
+    if run_helper.is_run_name(name):
+        match = True
+    elif job_helper.is_job_id(name):
+        match = True
+    else:
+        match = False
 
-   return match
+    return match
 
 def get_xt_objects_from_cmd_piping():
     objects = []
@@ -469,6 +475,14 @@ def get_explicit_options():
     '''
     return explict_options
 
+def set_explicit_options(options, additive=True):
+    global explict_options
+    
+    if additive:
+        explict_options.update(options)
+    else:
+        explict_options = options
+
 class Dispatcher():
     def __init__(self, impl_dict, config, preprocessor=None):
         self.impl_dict = impl_dict
@@ -477,9 +491,20 @@ class Dispatcher():
         self.cmd_words = None
         self.dispatch_cmd = None
         self.cmd_info = None
-        global commands_by_name, current_dispatcher
+
+        global commands_by_name, current_dispatcher, dispatch_counter, commands, save_commands
+
         commands_by_name = build_commands()
         current_dispatcher = self
+        dispatch_counter += 1
+
+        if dispatch_counter > 2:
+            if not save_commands:
+                # first time thru, save a copy of the commands
+                save_commands = copy.deepcopy(commands)
+            else:
+                # restore from saved copy
+                commands = copy.deepcopy(save_commands)
 
     def build_xt_cmd_from_dict(self, cmd, arguments, options, arg_dict):
         xt_cmd = cmd
