@@ -13,7 +13,7 @@ import numpy as np
 from fnmatch import fnmatch
 from collections import OrderedDict
 
-from .console import console
+from xtlib.console import console
 
 from xtlib import qfe
 from xtlib import utils
@@ -21,10 +21,9 @@ from xtlib import errors
 from xtlib import constants
 
 class ReportBuilder():
-    def __init__(self, config, store, client):
+    def __init__(self, config=None, store=None):
         self.config = config
         self.store = store
-        self.client = client
         self.user_col_args = {}
 
     def wildcard_matches_in_list(self, wc_name, name_list, omits):
@@ -203,14 +202,20 @@ class ReportBuilder():
 
     def flatten_records(self, records, sort_col, args):
         '''
-        pull out the cols specified by user, flattening nested props to their dotted names.
+        pull out the AVAILABLE or USER SPECIFIED columns, flattening nested props to their dotted names.
         '''
-         # build avail col list based on final set of filtered records
 
-        actual_cols, user_col_args  = self.get_actual_and_user_cols(records, args)
+        # build avail col list based on final set of filtered records
+        avail_cols, actual_cols, user_col_args  = self.get_avail_actual_user_cols(records, args)
 
         # flatten each record's nested columns
-        records = [self.extract_actual_cols(rec, actual_cols) for rec in records if rec]
+        available = utils.safe_value(args, "available")
+
+        if available:
+            records = [self.extract_actual_cols(rec, avail_cols) for rec in records if rec]
+        else:
+            records = [self.extract_actual_cols(rec, actual_cols) for rec in records if rec]
+
         return records
 
     def extract_actual_cols(self, record, actual_cols):
@@ -379,7 +384,7 @@ class ReportBuilder():
 
         return requested_list
 
-    def get_actual_and_user_cols(self, records, args):
+    def get_avail_actual_user_cols(self, records, args):
         col_dict = OrderedDict()
         for sr in records:
             if "metric_names" in sr:
@@ -399,7 +404,7 @@ class ReportBuilder():
         user_col_args = self.build_user_col_args(user_columns)
 
         actual_cols, user_col_args = self.get_requested_cols(user_col_args, avail_list)
-        return actual_cols, user_col_args
+        return avail_list, actual_cols, user_col_args
 
     def build_report(self, records, report_type, args):
         ''' build a tabular report of the records, or export to a table, as per args.  
@@ -410,7 +415,7 @@ class ReportBuilder():
         was_exported = False
         max_column_width = utils.safe_value(args, "max_width")
 
-        avail_list, user_col_args  = self.get_actual_and_user_cols(records, args)
+        avail_list, actual_cols, user_col_args  = self.get_avail_actual_user_cols(records, args)
 
         # store for later use
         self.user_col_args = user_col_args
@@ -446,7 +451,10 @@ class ReportBuilder():
         # write column header
         header = ""
         for col in col_list:
-            header += col + sep_char
+            if header == "":
+                header = col
+            else:
+                header += sep_char + col
         lines.append(header)
 
         # write value rows
@@ -457,7 +465,13 @@ class ReportBuilder():
                 value = record[col] if col in record else ""
                 if value is None:
                     value = ""
-                line += str(value) + sep_char
+                else:
+                    value = str(value)
+
+                if line == "":
+                    line = value
+                else:
+                    line += sep_char + value
     
             lines.append(line)
 
@@ -554,7 +568,7 @@ class ReportBuilder():
             text = "%.*e" % (significance-1, value)
         return text
 
-    def build_formatted_table(self, records, avail_cols, col_list=None, max_col_width=None, 
+    def build_formatted_table(self, records, avail_cols=None, col_list=None, max_col_width=None, 
         report_type="run-reports", group_by=None, number_groups=False):
         '''
         builds a nicely formatted text table from a set of records.
@@ -567,18 +581,31 @@ class ReportBuilder():
         time_col_names = ["created", "started", "ended", "time"]
         duration_col_names = ["duration", "queued"]
 
+        if not avail_cols:
+            avail_cols = list(records[0].keys()) if records else []
         #console.print("self.user_col_args=", self.user_col_args)
 
-        if not max_col_width:
-            max_col_width = int(self.config.get(report_type, "max-width"))    
+        if self.config:
+            if not max_col_width:
+                max_col_width = int(self.config.get(report_type, "max-width"))    
+                
+            precision = self.config.get(report_type, "precision")
+            significance = self.config.get(report_type, "significance")
+            max_fixed_length = self.config.get(report_type, "max-fixed-length")
             
-        precision = self.config.get(report_type, "precision")
-        significance = self.config.get(report_type, "significance")
-        max_fixed_length = self.config.get(report_type, "max-fixed-length")
-        
-        uppercase_hdr_cols = self.config.get(report_type, "uppercase-hdr")
-        right_align_num_cols = self.config.get(report_type, "right-align-numeric")
-        truncate_with_ellipses = self.config.get(report_type, "truncate-with-ellipses")
+            uppercase_hdr_cols = self.config.get(report_type, "uppercase-hdr")
+            right_align_num_cols = self.config.get(report_type, "right-align-numeric")
+            truncate_with_ellipses = self.config.get(report_type, "truncate-with-ellipses")
+        else:
+            # default when running without config file
+            if not max_col_width:
+                max_col_width = 35
+            precision = 3
+            significance = 2
+            max_fixed_length = 7
+            uppercase_hdr_cols = True
+            right_align_num_cols = True
+            truncate_with_ellipses = True
 
         if not col_list:
             col_list = avail_cols
@@ -861,4 +888,19 @@ class ReportBuilder():
 
             groups[group].append(rec)
 
-        return groups
\ No newline at end of file
+        return groups
+
+if __name__ == "__main__":
+    # test out simple use of ReportBuilder
+    builder = ReportBuilder()
+
+    records = []
+    for i in range(1, 11):
+        age = np.random.randint(2*i)
+        income = np.random.randint(10000*i)
+        record = {"name": "roland" + str(i), "age": age, "income": income}
+        records.append(record)
+
+    text, _ = builder.build_formatted_table(records)
+    print(text)
+
