@@ -231,6 +231,7 @@ class Run():
         return child_run
 
     def close(self):
+        
         if self.xt_logging and self.direct_run and self.store and self.context:
             context = self.context
             status = "completed"
@@ -246,6 +247,8 @@ class Run():
                 capture_files=context.after_upload, job_id=context.job_id, is_parent = True, node_id=node_id, 
                 run_index=None)
 
+        self.store.mongo.on_run_close()
+
         if self.train_writer:
             self.train_writer.close()
             self.test_writer.close()
@@ -295,14 +298,6 @@ class Run():
         if self.is_aml and self.aml_logging:
             self.aml_run.log_row("hparams", **data_dict)
 
-    # for simplicity, only support log_metrics
-    # def log_metric(self, name, value):
-    #     if self.store and self.xt_logging:
-    #         self.store.log_run_event(self.ws_name, self.run_name, "metrics", {name: value}, is_aml=self.is_aml)
-
-    #     if self.is_aml and self.aml_logging:
-    #         self.aml_run.log(name, value)
-
     def log_metrics(self, data_dict, step_name=None, stage=None):
         #console.print("log_metrics: self.store=", self.store, ", xt_logging=", self.xt_logging)
 
