@@ -17,6 +17,14 @@ from xtlib import store_utils
 
 from xtlib.console import console
 
+run_info_props = {"_id": 1, "app_name": 1, "box_name": 1, "compute": 1, "create_time": 1, "description": 1, "exper_name": 1,
+    "from_ip": 1, "from_computer_name": 1, "is_child": 1, "is_parent": 1, "is_outer": 1, "job_id": 1, 
+    "node_index": 1, "path": 1, "repeat": 1, "script": 1, "run_name": 1, "run_num": 1, "run_guid": 1,
+    "search_style": 1, "search_type": 1, "service_type": 1, "sku": 1, "username": 1, "ws_name": 1, 
+    "xt_build": 1, "xt_version": 1}
+
+run_stats_props = {"status": 1, "last_time": 1, "exit_code": 1, "restarts": 1, "end_time": 1, "end_id": 1, "tags": 1}
+
 def expand_run_list(store, mongo, workspace, name_list):
     '''
     args:
@@ -116,15 +124,14 @@ def set_run_tags(store, mongo, name_list, tag_list, workspace, fd, clear):
         matched_count = 0
 
         if len(run_list)==1 and run_list[0] == "*":
-            # update all records
-            filter = {}
+            # update ALL records in ws
+            all_ws_runs = mongo.get_info_for_runs(workspace, {"ws_name": workspace}, {"_id": 1})
+            run_list = [run["_id"] for run in all_ws_runs]
         else:
-            # convert simple run_name into id required by storage vx
+            # convert simple run_name into id required by storage version
             run_list = [store_utils.make_id(workspace, run) for run in run_list]
-            filter = {"run_name": {"$in": run_list}}
 
-            result = mongo.update_runs_from_filter(workspace, filter, fd, clear=clear, upsert=False)
-            matched_count = result.matched_count
+        matched_count = mongo.update_run_stats_from_list(workspace, run_list, fd, clear=clear, upsert=False)
 
         # for run in run_list:
         #     mongo.update_run_info(workspace, run, fd, clear=clear, upsert=False)  
@@ -141,7 +148,8 @@ def list_run_tags(store, mongo, ws_name, name_list, tag_list):
 
     if run_list:
         filter_dict = {"run_name": {"$in": run_list}}
-        fields_dict = {"tags": 1}
+        #fields_dict = {"tags": 1}
+        fields_dict = {"run_stats": 1}
 
         records = mongo.get_info_for_runs(ws_name, filter_dict, fields_dict)
         for record in records:
@@ -296,7 +304,6 @@ def build_run_filter_dict(ws_name, run_list, user_to_actual, builder, args):
             # or all of fany conditions together
             fd["$or"] = fany_list
 
-    #fd["is_parent"] = True
     return fd
 
 def extract_dotted_cols(records, prefix):
@@ -312,6 +319,21 @@ def extract_dotted_cols(records, prefix):
 
     return list(nd.keys())
 
+def get_different_cols(records):
+    # first pass, gather last value of each column seen
+    col_values = {}
+    for rd in records:
+        col_values.update(rd)
+
+    # second pass, build dict of diff columns
+    diff_cols = {}
+    for rd in records:
+        for key, value in col_values.items():
+            if not key in rd or rd[key] != value:
+                diff_cols[key] = True
+
+    return list(diff_cols)
+
 def get_filtered_sorted_limit_runs(store, config, show_gathering, col_dict=None, preserve_order=False, 
         col_names_are_external=True, flatten_records=True, args=None):
     # JIT import (workaround cyclic imports)
@@ -340,7 +362,7 @@ def get_filtered_sorted_limit_runs(store, config, show_gathering, col_dict=None,
     user_to_actual, std_cols_desc = get_run_property_dicts()        
     actual_to_user = {value: key for key, value in user_to_actual.items()}
 
-    builder = ReportBuilder(config, store, client=None)
+    builder = ReportBuilder(config, store)
 
     # get list of specified runs
     pure_run_list, actual_ws = expand_run_list(store, mongo, workspace, run_list)
@@ -385,7 +407,10 @@ def order_runs_by_user_list(runs, user_list):
     new_records = [nr for nr in new_records if isinstance(nr, dict)]
     return new_records
 
-def list_runs(store, config, args):
+def list_runs(store, config, args, compare=False):
+
+    if compare:
+        args["columns"] = ["job", "run", "hparams.*"]
 
     records, using_default_last, user_to_actual, available, builder, last, std_cols_desc = \
         get_filtered_sorted_limit_runs(store, config, show_gathering=True, 
@@ -403,6 +428,11 @@ def list_runs(store, config, args):
         for line in lines:
             console.print(line)
     else:            
+        if compare:
+            # filter cols to those that are different
+            diff_cols = get_different_cols(records)
+            args["columns"] = diff_cols 
+
         #avail_list = list(user_to_actual.keys())
         lines, row_count, was_exported = builder.build_report(records, report_type="run-reports", args=args)
 
@@ -589,16 +619,18 @@ def validate_run_name(store, ws, run_name, error_if_invalid=True, parse_only=Fal
                 return None, None, None
     return ws, run_name, ws + "/" + run_name
 
-def build_metrics_sets(records, steps=None, merge=False, metrics=None, timebase=None):
+def build_metrics_sets(records, steps=None, merge=False, metrics=None, timebase=None, cleanup=True):
     '''
-    builds a collection of metrics sets.  A metric set is a set of metrics that have been reported together 
-    in the run log.  Here 'together' means as a single log entry.
+    Args:
+        records: the set of dict records of a run log.
+        steps: an optional list of step values to filter by (return only matching step records).
+        merge: when True, merge all datasets into a single one
+        metrics: list of specific metric names to extract
+        cleanup: when True, restarts are detected and their older records are removed 
+
+    Processing:
+        We process all "metrics" event log records, grouping each by their property names.  Each group is a metric set.
 
-    args:
-        'records' is the set of dict records of a run log.
-        'steps' is an optional set of steps to filter by.
-        'merge': when True, merge all datasets into a single one
-        'metrics': list of specific metric names to extract
     '''
     # first step: put each metric into their own set (with time-stamped records)
     metric_sets_by_keys = {}
@@ -654,21 +686,22 @@ def build_metrics_sets(records, steps=None, merge=False, metrics=None, timebase=
             elif "epoch" in dd:
                 step_name = "epoch"
 
-        if steps and step_name:
-            # filter this record (skip if neq next_step)
+        if step_name:
             step_value = dd[step_name]
 
-            while step_value > next_step:
-                # compute next step
-                step_index += 1
-                if step_index < len(steps):
-                    next_step = steps[step_index]
-                else:
-                    # found all specified steps
-                    break
+            if steps:
+                # filter this record (skip if neq next_step)
+                while step_value > next_step:
+                    # compute next step
+                    step_index += 1
+                    if step_index < len(steps):
+                        next_step = steps[step_index]
+                    else:
+                        # found all specified steps
+                        break
 
-            if step_value < next_step:
-                continue
+                if step_value < next_step:
+                    continue
 
         # add time to record
         arrow_str = log_dict["time"]
@@ -727,8 +760,67 @@ def build_metrics_sets(records, steps=None, merge=False, metrics=None, timebase=
             df = {"keys": json.loads(keys_str), "records": records}
             metric_sets.append(df)
 
+    if cleanup and step_name:
+        for ms in metric_sets:
+            if has_restarts(ms["records"], step_name):
+                ms["records"] = cleanup_metric_set(ms["records"], step_name)
+                assert not has_restarts(ms["records"], step_name)
+
     return metric_sets
 
+def has_restarts(orig_records, step_name):
+    '''
+    Return True if log metric records have 1 or more restarts.
+    '''
+    last_step = None
+    restart_count = 0
+
+    # process list backwards
+    records = list(orig_records)
+    records.reverse()
+
+    for dd in records:
+        step = dd[step_name]
+        if not last_step or step < last_step:
+            last_step = step
+        else:
+            # RESTART detected
+            restart_count += 1
+            #console.print("skipping step: {}".format(step))
+
+    return restart_count > 0
+
+def cleanup_metric_set(orig_records, step_name):
+    '''
+    Processing:
+        Remove out of order records (caused by run restarts).
+    '''
+    last_step = None
+    new_records = []
+    restart_count = 0
+
+    # process list backwards
+    records = list(orig_records)
+    records.reverse()
+
+    for dd in records:
+        step = dd[step_name]
+        if not last_step or step < last_step:
+            new_records.append(dd)
+            last_step = step
+        else:
+            # RESTART detected
+            restart_count += 1
+            #console.print("skipping step: {}".format(step))
+
+    if restart_count:
+        console.diag("  restarts removed: {}".format(restart_count))
+
+    # need to reverse the new list
+    new_records.reverse()
+
+    return new_records
+
 def get_int_from_run_name(run_name):
     id = float(run_name[3:])*100000
     id = int(id)
@@ -838,13 +930,15 @@ def get_run_number(name):
         elif "." in part:
             parent, child = part.split(".", 1)
             parent = int(parent)
-            child = int(child)
+            child = 1 + int(child)
         else:
             parent = int(part)
 
         # allow for 1M children, 1M nodes
         oneM = 1000*1000
-        run_num = oneM*oneM*int(parent) + oneM*int(node) + int(child)
+
+        # sort such that all parent (node runs) come before all child runs
+        run_num = oneM*oneM*int(parent) + oneM*int(child) + int(node)
         
         return run_num
 
