@@ -358,22 +358,63 @@ class Runner():
 
         return runs_by_box, last_run
 
+    def adjust_script_dir(self, cmd_parts):
+        '''
+        NOTE: cmd_parts is modified directly.
+        '''
+
+        script_dir = None    # default to the current directory
+        found_script = False
+
+        parts = cmd_parts
+        dest_linux = not self.backend.gen_for_windows
+
+        for i, part in enumerate(parts):
+
+            if found_script and part.startswith("@"):
+                part = part[1:]   # remove the "@"
+                if os.path.isfile(part):
+                    # ensure the slashes in the path match targeted OS 
+                    parts[i] = "@" + file_utils.fix_slashes(part, is_linux=dest_linux)
+
+            elif not part.startswith("-"):
+                if os.path.isfile(part):
+                    # ensure the slashes in the path match targeted OS 
+                    fixed_path = file_utils.fix_slashes(part, is_linux=dest_linux)
+                    parts[i] = fixed_path
+
+                    script_dir = os.path.dirname(fixed_path)
+                    found_script = True
+
+        if not script_dir:
+            script_dir = "."
+
+        return script_dir
+
     def remove_script_dir_from_parts(self, cmd_parts):
         '''
         NOTE: cmd_parts is modified directly.
         '''
 
         script_dir = "."    # default to the current directory
+        found_script = False
 
         parts = cmd_parts
         for i, part in enumerate(parts):
-            path = os.path.realpath(part)
-            if os.path.isfile(path):
-                script_dir = os.path.dirname(path)
 
-                # remove the path from the script 
-                parts[i] = os.path.basename(path)
-                break
+            if found_script and part.startswith("@"):
+                part = part[1:]   # remove the "@"
+                if os.path.isfile(part):
+                    # remove the path from the arg file 
+                    parts[i] ="@" + os.path.basename(part)
+            elif not part.startswith("-"):
+                path = os.path.realpath(part)
+                if os.path.isfile(path):
+                    script_dir = os.path.dirname(path)
+
+                    # remove the path from the script 
+                    parts[i] = os.path.basename(path)
+                    found_script = True
 
         return script_dir
 
@@ -506,8 +547,10 @@ class Runner():
 
         code_upload = args["code_upload"]
 
-        # this step should always be done so that script_dir is removed from cmd_parts
-        script_dir = self.remove_script_dir_from_parts(cmd_parts)
+        # new design: script dir must be relative and we don't touch it
+        # # this step should always be done so that script_dir is removed from cmd_parts
+        # script_dir = self.remove_script_dir_from_parts(cmd_parts)
+        script_dir = self.adjust_script_dir(cmd_parts)
 
         if code_upload:
             for i, code_dir in enumerate(code_dirs):
@@ -515,7 +558,10 @@ class Runner():
                 if "$scriptdir" in code_dir:
                     code_dir = code_dir.replace("$scriptdir", script_dir)
 
-                if "::" in code_dir:
+                if "==>" in code_dir:
+                    code_dir, dest_dir = code_dir.split("==>")
+                elif "::" in code_dir:
+                    # "::" is legacy version of "==>"
                     code_dir, dest_dir = code_dir.split("::")
                 else:
                     dest_dir = "."
@@ -580,6 +626,9 @@ class Runner():
         # pass to everyone
         args["bootstrap_dir"] = bootstrap_dir
 
+        # parse arguments before paths are removed
+        job_hparams = self.build_job_hparams(cmd_parts)
+
         if fake_submit:
             script_dir = snapshot_dir
         else:
@@ -669,7 +718,7 @@ class Runner():
             box_name = boxes[i]
 
             # generate a box secret for talking to XT controller for this node
-            box_secret =  str(uuid.uuid4()) if remote_control else ""
+            box_secret = str(uuid.uuid4()) if remote_control else ""
 
             # build runs for box_name
             run_data = self.build_first_run_for_node(i, run_count, total_run_count, 
@@ -845,6 +894,9 @@ class Runner():
                 # CONNECT_INFO: embedded UPDATABLE props (2)
                 "connect_info_by_node": {}, 
                 "secrets_by_node": secrets_by_node,  
+
+                # job-detectable HPARAMS
+                "hparams": job_hparams,
             }
 
             self.store.log_job_info(workspace, job_id, dd)
@@ -862,9 +914,95 @@ class Runner():
         # return values for API support (X)
         return cmds, run_specs, using_hp, using_aml_hparam, sweeps_text, compute_def, job_id 
 
+    def get_cmd_line_args_from_file(self, fn):
+        with open(fn, "rt") as infile:
+            text = infile.read()
+            lines = text.split("\n")
+
+        arg_parts = []
+
+        for line in lines:
+            line = line.strip()
+            if line and not line.startswith("#"):
+                # remove comments from line with arg(s)
+                if "#" in line:
+                    line = line.split("#", 1)[0]
+
+                line = line.strip()
+                if line:
+                    parts = line.split()
+                    arg_parts += parts
+
+        hd = self.parse_script_args(arg_parts)
+        return hd
+
+    def build_job_hparams(self, cmd_parts):
+        hparams = None
+
+        # make copy to modify
+        first_arg_index = None
+
+        # first index of first script or app arg
+        if cmd_parts[0].startswith("python"):
+
+            # look for python options before script name
+            for i, part in enumerate(cmd_parts[1:]):
+                if not part.startswith("-"):
+                    # found script at i+1
+                    first_arg_index = i+2
+                    break
+
+        else:
+            first_arg_index = 1
+
+        if first_arg_index:
+            pending_name = None
+
+            args = cmd_parts[first_arg_index:]
+            hparams = self.parse_script_args(args)
+
+        return hparams
+
+    def parse_script_args(self, args):
+        hparams = {}
+        pending_name = None
+
+        for arg in args:
+            '''
+            look for arg/value in 3 forms:
+                - arg=value
+                - arg
+                - arg value  (value is in following arg)
+            '''
+            if arg.startswith("-"):
+                # strip the leading dashes
+                arg = arg.lstrip("-")
+
+                # found an arg name
+                if "=" in arg:
+                    name, value = arg.split("=")
+                    hparams[name] = value
+                    pending_name = None
+                else:
+                    # for now, treat as option without value
+                    hparams[arg] = None
+                    pending_name = arg
+            elif arg.startswith("@"):
+                fn = arg[1:]
+                ad = self.get_cmd_line_args_from_file(fn)
+                hparams.update(ad)
+            else:
+                # is this a value for a pending name?
+                if pending_name:
+                    hparams[pending_name] = arg
+                    pending_name = None
+
+        return hparams
+
     def fixup_script_in_cmd(self, cmd):
         cmd_parts = utils.cmd_split(cmd)
-        self.remove_script_dir_from_parts(cmd_parts)
+        
+        self.adjust_script_dir(cmd_parts)
 
         # add "-u" for python cmds
         if len(cmd_parts) > 1 and cmd_parts[0].startswith("python") and cmd_parts[1] != "-u":
@@ -1154,4 +1292,4 @@ def update_compute_def_from_cmd_options(compute_def, hold=False):
     if hold and "cluster" in compute_def and "queue" not in explict_opts:
         compute_def["queue"] = "interactive"
 
-            
\ No newline at end of file
+            
