@@ -18,6 +18,7 @@ from xtlib import console
 from xtlib import constants
 from xtlib import job_helper
 from xtlib import run_helper
+from xtlib import report_builder
 
 DEFAULT_DB = "__default__"
 MONGO_INFO = "__mongo_info__"
@@ -57,7 +58,8 @@ class MongoDB2():
                 - whenever possible, we use a predictable _id (since updates must specify the shard key)
     '''
     
-    def __init__(self, mongo_cs, db_name=None, ws_name=None, run_cache_dir=None, direct_mode=True):
+    def __init__(self, mongo_cs, db_name=None, ws_name=None, run_cache_dir=None, 
+        direct_mode=True, store=None, buffer_metrics=False):
 
         '''
         direct_mode: means we update data in mongo directly (from XT or the run) (vs. storge tracking)
@@ -73,8 +75,11 @@ class MongoDB2():
         self.mongo_conn_str = mongo_cs
         self.round_trip_count = 0         # stats for last query
         self.name = "mongo_v2"
+        self.store = store
+        self.buffer_metrics = buffer_metrics
+        self.metrics = {}
 
-        # controls for mongo stats and logging
+        # default controls for mongo stats and logging
         self.update_job_stats = True
         self.update_run_stats = True
         self.add_log_records = True
@@ -141,26 +146,28 @@ class MongoDB2():
         '''
 
         # this line will raise an exception if the ws_name is not known to mongo
-        dbx = self.get_db(ws_name)
+        exists = bool(self.get_workspace_infos({"_id": ws_name}))
+        if exists:
+            dbx = self.get_db(ws_name)
 
-        # delete RUN collections
-        self.delete_from_collection(dbx, ws_name, "run_info", "_id")
-        self.delete_from_collection(dbx, ws_name, "hparams", "_id")
-        self.delete_from_collection(dbx, ws_name, "metrics", "_id")
-        self.delete_from_collection(dbx, ws_name, "log_records", "_id")
-        self.delete_from_collection(dbx, ws_name, "run_stats", "_id")
+            # delete RUN collections
+            self.delete_from_collection(dbx, ws_name, "run_info", "_id")
+            self.delete_from_collection(dbx, ws_name, "hparams", "_id")
+            self.delete_from_collection(dbx, ws_name, "metrics", "_id")
+            self.delete_from_collection(dbx, ws_name, "log_records", "_id")
+            self.delete_from_collection(dbx, ws_name, "run_stats", "_id")
 
-        # delete JOB collections
-        self.delete_from_collection(dbx, ws_name, "job_info", "_id")
-        self.delete_from_collection(dbx, ws_name, "job_stats", "_id")
-        self.delete_from_collection(dbx, ws_name, "connect_info", "_id")
-        self.delete_from_collection(dbx, ws_name, "service_info_by_node", "_id")
-        self.delete_from_collection(dbx, ws_name, "child_runs", "_id")
+            # delete JOB collections
+            self.delete_from_collection(dbx, ws_name, "job_info", "_id")
+            self.delete_from_collection(dbx, ws_name, "job_stats", "_id")
+            self.delete_from_collection(dbx, ws_name, "connect_info", "_id")
+            self.delete_from_collection(dbx, ws_name, "service_info_by_node", "_id")
+            self.delete_from_collection(dbx, ws_name, "child_runs", "key")
 
-        dbx_default = self.get_db(db_name=DEFAULT_DB)
-        self.mongo_with_retries("delete_workspace_if_needed", lambda: dbx_default[WORKSPACES].delete_one( {"_id": ws_name} ))
+            dbx_default = self.get_db(db_name=DEFAULT_DB)
+            self.mongo_with_retries("delete_workspace_if_needed", lambda: dbx_default[WORKSPACES].delete_one( {"_id": ws_name} ))
 
-        return True
+        return exists
 
     def delete_from_collection(self, dbx, ws_name, collection_name, shard_key=None):
         '''
@@ -178,26 +185,33 @@ class MongoDB2():
 
                 # Cosmos DB doesn't support delete_many with shard keys, so we work around issue with bluk_delete
                 #self.mongo_with_retries("delete_from_collection", lambda: dbx[collection_name].delete_many( {shard_key: {"$in": shard_keys}} ))
-                self.limited_bulk_delete(dbx, collection_name, shard_keys)
+                self.limited_bulk_delete(dbx, collection_name, shard_keys, key_name=shard_key)
         else:
             # easy case; just delete by ws_name
             self.mongo_with_retries("delete_from_collection", lambda: dbx[collection_name].delete_many( {"ws_name": ws_name} ))
 
-    def limited_bulk_delete(self, dbx, collection_name, ids):
+    def limited_bulk_delete(self, dbx, collection_name, keys, key_name="_id"):
+        '''
+        Limit number of documents being deleted to a small avoid to avoid "request rate too large"
+        errors.  Since using shard keys with Cosmos, we can only target a single key (document) on 
+        each delete, so we use bulk_write to batch up a set of commands.
+        '''
+
         max_deletes = 10    # 25 gives "request rate is too large" errors 
         index = 0
 
-        while index < len(ids):
-            id_batch = ids[index:index+max_deletes]
+        while index < len(keys):
+            key_batch = keys[index:index+max_deletes]
 
             delete_cmds = []
-            for id in id_batch:
-                cmd = pymongo.operations.DeleteOne( {"_id": id} )
+            for key in key_batch:
+                cmd = pymongo.operations.DeleteOne( {key_name: key} )
                 delete_cmds.append(cmd)
 
             self.mongo_with_retries("limited_bulk_delete", lambda: dbx[collection_name].bulk_write( delete_cmds ))
 
-            index += len(id_batch)
+            index += len(key_batch)
+
 
     def create_workspace_if_needed(self, ws_name, db_name=None):
         fd = {"_id": ws_name}
@@ -299,9 +313,12 @@ class MongoDB2():
         retry_count = 25
         result = None
         started = time.time()
+        actual_call_time = None
 
         for i in range(retry_count):
             try:
+                actual_started = time.time()
+
                 result = mongo_cmd()
 
                 # most of the time, we want to ALSO want to retry building a record set from the cursor
@@ -312,6 +329,8 @@ class MongoDB2():
                         result = list(result)
                         if result:
                             result = result[0]
+
+                actual_call_time = time.time() - actual_started
                 break
             # watch out for these exceptions: AutoReconnect, OperationFailure (and ???)
             except BaseException as ex:   # pymongo.errors.OperationFailure as ex:
@@ -353,7 +372,9 @@ class MongoDB2():
 
         if not name in self.call_stats:
             self.call_stats[name] = []
-        self.call_stats[name].append(elapsed)
+
+        entry = {"elapsed": elapsed, "actual_call_time": actual_call_time, "retry_count": i}
+        self.call_stats[name].append(entry)
 
         #print("--> mongo call: {} (elapsed: {:.4f} secs)".format(name, elapsed))
         return result
@@ -364,69 +385,6 @@ class MongoDB2():
             if isinstance(value, dict):
                 assert name in exceptions
 
-    def update_job(self, jd):
-        ws_name = jd["ws_name"]
-        job_id = jd["job_id"]
-        total_runs = jd["run_count"]
-
-        # delete low-priority embedded info
-        utils.safe_delete(jd, "runs_by_box")
-
-        # delete obsolete embedded info
-        utils.safe_delete(jd, "active_runs")
-        utils.safe_delete(jd, "dynamic_runs_remaining")
-
-        # extract embedded info
-        service_info_by_node = utils.safe_delete(jd, "service_info_by_node")
-        runs_by_box = utils.safe_delete(jd, "runs_by_box")
-        connect_info_by_node = utils.safe_delete(jd, "connect_info_by_node")
-        secrets_by_node = utils.safe_delete(jd, "secrets_by_node")
-        child_runs_by_node = utils.safe_delete(jd, "child_runs_by_node")
-
-        # extract job stats
-        job_stats = {}
-        utils.safe_move(job_stats, jd, "job_status")
-        utils.safe_move(job_stats, jd, "completed_runs")
-        utils.safe_move(job_stats, jd, "error_runs")
-        utils.safe_move(job_stats, jd, "running_nodes")
-        utils.safe_move(job_stats, jd, "running_runs")
-
-        # hardcode some stats so that when processing job log, counts are correct
-        job_stats["total_runs"] = total_runs
-        job_stats["completed_runs"] = 0
-        job_stats["error_runs"] = 0
-        job_stats["running_nodes"] = 0
-        job_stats["running_runs"] = 0
-
-        self.add_id(job_stats, ws_name, job_id)
-        self.update_collection("job_stats", ws_name, job_stats)
-
-        # CHILD RUNS
-        if child_runs_by_node:
-            self.add_docs_by_node("child_runs", ws_name, job_id, child_runs_by_node)
-
-        # add to SERVICE_INFO_BY_NODE
-        if service_info_by_node:
-            self.add_id(service_info_by_node, ws_name, job_id)
-            self.update_collection("service_info_by_node", ws_name, service_info_by_node, is_flat=False)
-
-        # add to CONNECT_INFO
-        if connect_info_by_node or secrets_by_node:
-            # merge secrets into connect info
-            if connect_info_by_node:
-                for n1, n2 in zip(connect_info_by_node, secrets_by_node):
-                    connect_info_by_node[n1]["secret"] = secrets_by_node[n1]
-            else:
-                connect_info_by_node = {}
-                for node, secret in secrets_by_node.items():
-                    connect_info_by_node[node] = {"secret": secret}
-
-            self.add_doc_by_node("connect_info", ws_name, job_id, connect_info_by_node)
-
-        # add to JOB_INFO
-        self.add_id(jd, ws_name, job_id)
-        self.update_collection("job_info", ws_name, jd, flat_exceptions=["pool_info", "service_job_info"])
-
     def update_connect_info_by_node(self, ws_name, job_id, node_id, connect_info_dict):
         _id = "{}/{}/{}".format(ws_name, job_id, node_id)
         connect_info_dict["_id"] = _id
@@ -501,7 +459,14 @@ class MongoDB2():
     def get_collection_records(self, collection, ws_name, filter, fields=None):
         dbx = self.get_db(ws_name)
 
-        records = self.mongo_with_retries("get_collection_records", lambda: dbx[collection].find(filter, fields), return_records=True)
+        records = self.mongo_with_retries("get_collection_records", lambda: dbx[collection].find(filter, fields), 
+            return_records=True)
+
+        # experimental code: failover to storage if records not found
+        if not records and collection == "log_records":
+            id_list = filter["key"]["$in"]
+            records = self.store.get_log_records_for_runs(ws_name, id_list)
+
         return records
         
     def update_collection(self, collection_name, ws_name, dd, dbx=None, is_flat=True, flat_exceptions=[]):
@@ -524,7 +489,8 @@ class MongoDB2():
         update = {"$set": dd}
         filter = {"_id": dd["_id"]}
 
-        self.mongo_with_retries("update_collection", lambda: dbx[collection_name].update_one(filter=filter, update=update, upsert=True) )
+        my_name = "update_collection__{}".format(collection_name)
+        self.mongo_with_retries(my_name, lambda: dbx[collection_name].update_one(filter=filter, update=update, upsert=True) )
             
     def update_sys_collection(self, collection_name, ws_name, dd, dbx=None):
         '''
@@ -580,6 +546,96 @@ class MongoDB2():
         next_id = record[path]
         return next_id
 
+    def create_mongo_job(self, jd):
+        ws_name = jd["ws_name"]
+        job_id = jd["job_id"]
+
+        self.update_job_info(ws_name, job_id, jd, update_primary=True)
+        
+    def update_job_info(self, ws_name, job_id, orig_dd, update_primary=False):
+        '''
+        Args:
+            - ws_name: name of the associated workspace
+            - job_id: name of the job being updated
+            - orig_dd: a dictionary of name/value pairs.  Can include nested: service_info_by_node,
+                connect_info, child_runs, and hparams.  Other props split between job_info and job_stats.
+            - update_primary: specifies if the job_info collection be updated
+
+        Processing:
+            This is the CORE function for updating job related information.  The following 
+            collections may be updated: job_info, job_stats, service_info_by_node,
+                connect_info, child_runs, and hparams.
+
+            In order for the job_info collection to be updated, update_primary must = True.
+        '''
+        jd = dict(orig_dd)    # make a copy to operator on
+        total_runs = jd["run_count"]
+
+        # delete low-priority embedded info
+        utils.safe_delete(jd, "runs_by_box")
+
+        # delete obsolete embedded info
+        utils.safe_delete(jd, "active_runs")
+        utils.safe_delete(jd, "dynamic_runs_remaining")
+
+        # extract embedded info
+        service_info_by_node = utils.safe_delete(jd, "service_info_by_node")
+        runs_by_box = utils.safe_delete(jd, "runs_by_box")
+        connect_info_by_node = utils.safe_delete(jd, "connect_info_by_node")
+        secrets_by_node = utils.safe_delete(jd, "secrets_by_node")
+        child_runs_by_node = utils.safe_delete(jd, "child_runs_by_node")
+        hparams = utils.safe_delete(jd, "hparams")
+
+        # extract job stats
+        job_stats = {}
+        utils.safe_move(job_stats, jd, "job_status")
+        utils.safe_move(job_stats, jd, "completed_runs")
+        utils.safe_move(job_stats, jd, "error_runs")
+        utils.safe_move(job_stats, jd, "running_nodes")
+        utils.safe_move(job_stats, jd, "running_runs")
+
+        # hardcode some stats so that when processing job log, counts are correct
+        job_stats["total_runs"] = total_runs
+        job_stats["completed_runs"] = 0
+        job_stats["error_runs"] = 0
+        job_stats["running_nodes"] = 0
+        job_stats["running_runs"] = 0
+
+        self.add_id(job_stats, ws_name, job_id)
+        self.update_collection("job_stats", ws_name, job_stats)
+
+        # CHILD RUNS
+        if child_runs_by_node:
+            self.add_docs_by_node("child_runs", ws_name, job_id, child_runs_by_node)
+
+        # add to SERVICE_INFO_BY_NODE
+        if service_info_by_node:
+            self.add_id(service_info_by_node, ws_name, job_id)
+            self.update_collection("service_info_by_node", ws_name, service_info_by_node, is_flat=False)
+
+        # add to CONNECT_INFO
+        if connect_info_by_node or secrets_by_node:
+            # merge secrets into connect info
+            if connect_info_by_node:
+                for n1, n2 in zip(connect_info_by_node, secrets_by_node):
+                    connect_info_by_node[n1]["secret"] = secrets_by_node[n1]
+            else:
+                connect_info_by_node = {}
+                for node, secret in secrets_by_node.items():
+                    connect_info_by_node[node] = {"secret": secret}
+
+            self.add_doc_by_node("connect_info", ws_name, job_id, connect_info_by_node)
+
+        # add to HPARAMS
+        if hparams:
+            self.add_id(hparams, ws_name, job_id)
+            self.update_collection("hparams", ws_name, hparams)
+
+        # add to JOB_INFO
+        if update_primary:
+            self.add_id(jd, ws_name, job_id)
+            self.update_collection("job_info", ws_name, jd, flat_exceptions=["pool_info", "service_job_info"])
+
     def create_mongo_run(self, orig_dd):
         '''
         Create or update run information in any/all of the 5 run collections.
@@ -666,20 +722,22 @@ class MongoDB2():
             metrics["_id"] = _id
             metrics["ws_name"] = ws_name
             update_doc = { "$set": metrics}
+
             self.mongo_with_retries("update_run_info", \
                 lambda: dbx["metrics"].update_one( {"_id": _id}, update_doc, upsert=True))
 
         # update LOG_RECORDS
-        if log_records:
+        if log_records and self.add_log_records:
             if isinstance(log_records, list):
                 # list of log records (likely called from import)
                 self.add_log_record_bulk_limited(ws_name, run_name, log_records)
             else:
                 self.add_log_record(ws_name, run_name, log_records)
 
-    def process_run_event(self, ws_name, run_name, event, dd, record_dict):
+    def process_run_event(self, ws_name, run_name, event, orig_dd, record_dict):
         run_stats = {}
         completed = False
+        dd = dict(orig_dd)
 
         if event == "created":
             #self.create_mongo_run(dd)
@@ -709,16 +767,34 @@ class MongoDB2():
             self.update_collection("hparams", ws_name, dd)
 
         elif event == "metrics":
+            metrics_dd = dict(dd)
             self.add_id(dd, ws_name, run_name)
-            self.update_collection("metrics", ws_name, dd)
+
+            if self.buffer_metrics:
+                #console.print("buffering metrics: {}".format(dd))
+                self.metrics.update(dd)
+                #console.print("after metrics update: self.metrics=", self.metrics)
+
+            else:
+                self.update_collection("metrics", ws_name, dd)
 
         if run_stats:
             self.add_id(run_stats, ws_name, run_name)
             self.update_collection("run_stats", ws_name, run_stats)
 
-        self.add_log_record(ws_name, run_name, record_dict)
+        if self.add_log_records:
+            self.add_log_record(ws_name, run_name, record_dict)
+
         return completed
 
+    def on_run_close(self):
+        if self.metrics:
+            # at end of run, write out the buffered metrics record
+            console.print("writing METRICS at end of run: {}".format(self.metrics))
+            ws_name = self.metrics["_id"].split("/")[0]
+            
+            self.update_collection("metrics", ws_name, self.metrics)
+
     def add_log_record(self, ws_name, run_name, orig_dd):
         '''
         add single log record dict (dd) to LOG_RECORDS collection
@@ -759,7 +835,7 @@ class MongoDB2():
 
             index += len(log_batch)
 
-    def update_runs_from_filter(self, ws_name, filter, dd, clear=False, upsert=True):
+    def update_run_stats_from_list(self, ws_name, id_list, dd, clear=False, upsert=True):
         '''
         Update 1 or more run_stats documents from a single set of dd properties.
         Mostly used to update tags on runs.
@@ -771,15 +847,59 @@ class MongoDB2():
         else:
             update_doc = { "$set": dd}
 
-        # ensure we filter by ws_name to target correct runs
-        if not "ws_name" in filter:
-            filter["ws_name"] = ws_name
+        #update_filter = {"_id": {"$in": id_list}}
 
-        # update, create prop if needed
-        result = self.mongo_with_retries("update_runs_from_filter", \
-            lambda: dbx["run_stats"].update_many( filter, update_doc, upsert=upsert) )
+        # # update, create prop if needed
+        # result = self.mongo_with_retries("update_run_stats_from_list", \
+        #     lambda: dbx["run_stats"].update_many( update_filter, update_doc, upsert=upsert) )
 
-        return result
+        # seems mongo requires us to specify a single shard key per query, so let's
+        # do this one at a time
+
+        matched_count = 0
+
+        for id in id_list:
+            update_filter = {"_id": id}
+
+            result = self.mongo_with_retries("update_run_stats_from_list", \
+                lambda: dbx["run_stats"].update_one( update_filter, update_doc, upsert=upsert) )
+
+            matched_count += result.matched_count
+
+        return matched_count
+
+    def update_job_stats_from_list(self, ws_name, id_list, dd, clear=False, upsert=True):
+        '''
+        Update 1 or more job_stats documents from a single set of dd properties.
+        Mostly used to update tags on jobs.
+        '''
+        dbx = self.get_db(ws_name)
+
+        if clear:
+            update_doc = { "$unset": dd}
+        else:
+            update_doc = { "$set": dd}
+
+        #update_filter = {"_id": {"$in": id_list}}
+
+        # update job_stats
+        #result = self.mongo_with_retries("update_job_stats_from_list", \
+        #    lambda: dbx["job_stats"].update_many( update_filter, update_doc, upsert=upsert) )
+
+        # seems mongo requires us to specify a single shard key per query, so let's
+        # do this one at a time
+
+        matched_count = 0
+
+        for id in id_list:
+            update_filter = {"_id": id}
+
+            result = self.mongo_with_retries("update_job_stats_from_list", \
+                lambda: dbx["job_stats"].update_one( update_filter, update_doc, upsert=upsert) )
+
+            matched_count += result.matched_count
+
+        return matched_count
 
     def process_job_events(self, ws_name, job_id, event_records):
         completed = False
@@ -1019,19 +1139,51 @@ class MongoDB2():
 
     def print_call_stats(self):
         total_count = 0
-        total_time = 0
+        total_elapsed = 0
+        total_actual = 0
         total_calls = 0
+        total_retries = 0
 
-        for name, stats in self.call_stats.items():
-            mean = np.mean(stats)
-            print("  {}x {}: avg={:.4f}".format(len(stats), name, mean))
+        # build a list of records that we can generate a report from
+        records = []
 
-            total_calls += len(stats)
-            total_time += np.sum(stats)
+        for name, entries in self.call_stats.items():
+            elapsed_list = [entry["elapsed"] for entry in entries]
+            actual_call_list = [entry["actual_call_time"] for entry in entries]
+            retry_count_list = [entry["retry_count"] for entry in entries]
+
+            mean_elapsed = np.mean(elapsed_list)
+            mean_actual_call = np.mean(actual_call_list)
+            mean_retry = np.mean(retry_count_list)
+
+            record = {}
+            record["calls"] = len(elapsed_list)
+            record["name"] = name
+            record["mean_elapsed"] = mean_elapsed
+            record["mean_actual"] = mean_actual_call
+            record["mean_retry"] = mean_retry
+
+            records.append(record)
+
+            # name = (name + ":").ljust(45)
+            # print("  {}x {}\t AVERAGE elapsed={:.2f},\t actual={:.2f},\t retries={:.1f}".format(len(elapsed_list), name,
+            #     mean_elapsed, mean_actual_call, mean_retry))
+
+            total_calls += len(elapsed_list)
+            total_elapsed += np.sum(elapsed_list)
+            total_actual += np.sum(actual_call_list)
+            total_retries += np.sum(retry_count_list)
             total_count += 1
 
-        print()
-        print("  {}x {}: total={:.4f} secs".format(total_calls, "CALLS", total_time))
+        builder = report_builder.ReportBuilder()
+
+        text, row_count = builder.build_formatted_table(records)
+
+        # print report
+        print(text)
+
+        print("  {}x {}: TOTAL elapsed={:.2f} secs, actual={:.2f} secs, retries={:,}". \
+            format(total_calls, "CALLS", total_elapsed, total_actual, total_retries))
         print()
 
     def get_info_for_collection(self, ws_name, collection_name, filter_dict, fields_dict=None):
@@ -1135,6 +1287,23 @@ class MongoDB2():
 
         return ids
         
+    def filter_uses_tags(self, name, value):
+        uses_tags = name.startswith("tags.")
+        if not uses_tags:
+            if isinstance(value, list):
+
+                # value is a list of dict's
+                for dd in value:
+                    for key in dd.keys():
+                        if key.startswith("tags."):
+                            uses_tags = True
+                            break
+
+                    if uses_tags:
+                        break
+            
+        return uses_tags
+
     def get_filtered_sorted_collection(self, collections, primary_collection, ws_name, filter_dict, fields_dict, 
         sort_col, sort_dir, skip, first, count_records, buffer_size):
 
@@ -1170,6 +1339,10 @@ class MongoDB2():
                     # xxx_info or xxx_stats
                     c_filter[name] = value
                     del filter_dict[name]
+                elif self.filter_uses_tags(name, value) and "tags" in c_props:
+                    # xxx_info or xxx_stats
+                    c_filter[name] = value
+                    del filter_dict[name]
 
             if c_sorted:
                 # we have reached the last collection
@@ -1204,9 +1377,21 @@ class MongoDB2():
                     return []
 
         # now, get matching RUN INFO records, with needed sort/skip/first/last/fields
-        if total_ids:
+        # we need to merge total_ids (from previously processed collections) with filter_dict (filter for this collection)
+        if total_ids is not None:
             if "ws_name" in filter_dict:
                 del filter_dict["ws_name"]
+
+            if "_id" in filter_dict:
+                # set total_ids to the INTERSERCTION of total_ids and ids from filter_dict
+                value = filter_dict["_id"]
+                if isinstance(value, str):
+                    value = [value]
+                elif isinstance(value, dict):
+                    value = value["$in"]
+
+                total_ids = list(set(total_ids) & set(value)) 
+
             filter_dict["_id"] = {"$in": total_ids}
 
         if sort_col and not sort_col in filter_dict:
@@ -1228,7 +1413,9 @@ class MongoDB2():
 
             # this is where we can encounter "max message size" errors if we try to get all
             # records at once
-            cursor = cursor.batch_size(buffer_size)
+            if buffer_size:
+                cursor = cursor.batch_size(buffer_size)
+
             records = list(cursor)
         else:
             records = []
@@ -1247,25 +1434,18 @@ class MongoDB2():
             filter and sort runs (supporting filter and sort cols in related collections) and return job_id's.
         '''
 
-        run_info_props = {"_id": 1, "app_name": 1, "box_name": 1, "compute": 1, "create_time": 1, "description": 1, "exper_name": 1,
-            "from_ip": 1, "from_computer_name": 1, "is_child": 1, "is_parent": 1, "is_outer": 1, "job_id": 1, 
-            "node_index": 1, "path": 1, "repeat": 1, "script": 1, "run_name": 1, "run_num": 1, "run_guid": 1,
-            "search_style": 1, "search_type": 1, "service_type": 1, "sku": 1, "username": 1, "xt_build": 1, "xt_version": 1}
-
-        run_stats_props = {"status": 1, "last_time": 1, "exit_code": 1, "restarts": 1, "end_time": 1, "end_id": 1}
-
         collections = [
-            {"name": "run_info", "props": run_info_props, "sorted": 0},
-            {"name": "run_stats", "props": run_stats_props, "sorted": 0},
+            {"name": "run_info", "props": run_helper.run_info_props, "sorted": 0},
+            {"name": "run_stats", "props": run_helper.run_stats_props, "sorted": 0},
             {"name": "hparams", "props": None, "sorted": 0},
             {"name": "metrics", "props": None, "sorted": 0}
         ]
 
         # re-order collections so that the one being sorted is on the bottom
         if sort_col:
-            if sort_col in run_info_props:
+            if sort_col in run_helper.run_info_props:
                 collections[0]["sorted"] = 1
-            elif sort_col in run_stats_props:
+            elif sort_col.startswith("tags.") or sort_col in run_helper.run_stats_props:
                 collections[1]["sorted"] = 1
             elif sort_col.startswith("hparams."):
                 collections[2]["sorted"] = 1
@@ -1288,23 +1468,21 @@ class MongoDB2():
             filter and sort jobs (supporting filter and sort cols in related collections) and return job_id's.
         '''
 
-        job_info_props = {"_id": 1, "compute": 1, "concurrent": 1, "exper_name": 1, "hold": 1, "job_id": 1, "job_num": 1,
-            "job_guid": 1, "job_secret": 1, "node_count": 1, "primary_metric": 1, "run_count": 1, "repeat": 1, 
-            "schedule": 1, "search_type": 1, "search_style": 1, "username": 1, "xt_cmd": 1, "started": 1}
-
-        job_stats_props = {"job_status": 1, "completed_runs": 1, "error_runs": 1, "running_nodes": 1, "total_runs": 1}
-
         collections = [
-            {"name": "job_info", "props": job_info_props, "sorted": 0},
-            {"name": "job_stats", "props": job_stats_props, "sorted": 0},
+            {"name": "job_info", "props": job_helper.job_info_props, "sorted": 0},
+            {"name": "job_stats", "props": job_helper.job_stats_props, "sorted": 0},
+            {"name": "hparams", "props": None, "sorted": 0},
         ]
 
         # re-order collections so that the one being sorted is on the bottom
         if sort_col:
-            if sort_col in job_info_props:
+            if sort_col in job_helper.job_info_props:
                 collections[0]["sorted"] = 1
-            elif sort_col in job_stats_props:
+            elif sort_col.startswith("tags.") or sort_col in job_helper.job_stats_props:
                 collections[1]["sorted"] = 1
+            elif sort_col.startswith("hparams."):
+                collections[2]["sorted"] = 1
+                sort_col = sort_col.split(".", 1)[1]
 
             collections.sort(key=lambda i: i["sorted"])
 
@@ -1344,11 +1522,11 @@ class MongoDB2():
                     needed["log_records"] = value
                     del fields_dict[name]
 
-                elif name == "run_stats":
+                elif name == "run_stats" or name.startswith("tags.") or name in run_helper.run_stats_props:
                     needed["run_stats"] = value
                     del fields_dict[name]
 
-                elif name == "run_info":
+                elif name == "run_info" or name in run_helper.run_info_props:
                     needed["run_info"] = value
                     del fields_dict[name]
                     #fields_dict["_nosuchfield_"] = 0    # for all run_info fields to be returned
@@ -1361,11 +1539,38 @@ class MongoDB2():
                     needed["hparams"] = value
                     del fields_dict[name]
 
+                else:
+                    errors.internal_error("unrecognized RUN col_name: {}".format(name))
+
         if not fields_dict:
             fields_dict = None
 
         return needed, fields_dict
 
+    def get_all_runs(self, aggregator_dest, ws_name, job_or_exper_name, filter_dict=None, fields_dict=None, use_cache=True, 
+        fn_cache=None, batch_size=None):
+
+        '''
+        Args:
+            aggregator_dest: "job" or "experiment"
+            ws_name: name of workspace containing the job or experiment
+            job_or_exper_name: name of the aggregating job or experiment
+            filter_dict: dict of fields to filter with
+            fields_dict: dict of fields to return
+            use_cache: (currently not used)
+            fn_cache: (currently not used)
+            batch_size: used to limit the number of records returned on each round trip to the server
+        '''
+        if not filter_dict:
+            if aggregator_dest == "job":
+                filter_dict = {"job_id": job_or_exper_name}
+            elif aggregator_dest == "experiment":
+                filter_dict = {"exper_name": job_or_exper_name}
+
+        records = self.get_filtered_sorted_run_info(ws_name, filter_dict, fields_dict, buffer_size=batch_size)
+
+        return records
+
     def get_filtered_sorted_run_info(self, ws_name, filter_dict, fields_dict, sort_col=None, sort_dir=1, skip=None, 
             first=None, count_runs=False, buffer_size=50):
         '''
@@ -1433,12 +1638,13 @@ class MongoDB2():
         needed["connect_info"] = default
         needed["service_info_by_node"] = default
         needed["job_info"] = default
+        needed["hparams"] = default
 
         if fields_dict:
             orig_fd = dict(fields_dict)
 
             for name, value in orig_fd.items():
-                if name.startswith("job_stats"):
+                if name.startswith("job_stats") or name.startswith("tags.") or name in job_helper.job_stats_props:
                     needed["job_stats"] = value
                     del fields_dict[name]
 
@@ -1446,22 +1652,30 @@ class MongoDB2():
                     needed["connect_info"] = value
                     del fields_dict[name]
 
-                elif name == "service_info_by_node":
+                elif name in ["service_info_by_node", "runs_by_box"]:
                     needed["service_info_by_node"] = value
                     del fields_dict[name]
 
-                elif name == "job_info":
+                elif name == "job_info" or name in job_helper.job_info_props:
                     needed["job_info"] = value
                     del fields_dict[name]
                     #fields_dict["_nosuchfield_"] = 0    # for all job_info fields to be returned
 
+                elif name.startswith("hparams"):
+                    needed["hparams"] = value
+                    del fields_dict[name]
+
                 elif name == "all":
                     needed["job_stats"] = value
                     needed["connect_info"] = value
                     needed["service_info_by_node"] = value
                     needed["job_info"] = value
+                    needed["hparams"] = value
                     del fields_dict[name]
 
+                else:
+                    errors.internal_error("Unrecognized JOB col: {}".format(name))
+
         if not fields_dict:
             fields_dict = None
 
@@ -1486,9 +1700,7 @@ class MongoDB2():
         return records
 
     def get_info_for_jobs(self, ws_name, filter_dict, fields_dict=None):
-
         self.round_trip_count = 0
-
         needed_collections, fields_dict = self.calc_needed_job_collections(fields_dict)
 
         records = self.get_info_for_collection(ws_name, "job_info", filter_dict, fields_dict)
@@ -1518,6 +1730,12 @@ class MongoDB2():
             self.join_data(records, connect_info, embed_name="connect_info")
             self.round_trip_count += 1
 
+        # add HPARAMS
+        if needed["hparams"]:
+            hparams = self.get_collection_records("hparams", ws_name, {"_id": {"$in": ids}})
+            self.join_data(records, hparams, embed_name="hparams")
+            self.round_trip_count += 1
+
     def join_data(self, records, infos, info_key="_id", embed_name=None):
         # build a dict to quickly find records
         rd = {r["_id"]: r for r in records}
@@ -1556,7 +1774,7 @@ class MongoDB2():
                 record = rd[id]
 
                 # fixup log record
-                del log["_id"]
+                del log["_id"] 
                 del log["key"]
 
                 if "data" in log:
@@ -1602,6 +1820,21 @@ class MongoDB2():
 
         return run_name
 
+    def get_all_experiments_in_ws(self, ws_name):
+        dbx = self.get_db(ws_name)
+
+        # cannot get "distinct" command to work ("command not supported")
+        #cursor = db["__jobs__"].distinct("ws_name") 
+
+        records = self.mongo_with_retries("get_all_experiments_in_ws", lambda: \
+            dbx["job_info"].find({"ws_name": ws_name}, {"exper_name": 1}), return_records=True)
+
+        exper_names = [rec["exper_name"] for rec in records if "exper_name" in rec]
+        exper_names = list(set(exper_names))   # remove dups
+
+        console.diag("after get_all_experiments()")        
+        return exper_names        
+
 class MongoRunIndex():
     '''
     used by Controller to manage scheduling of child runs.
