@@ -12,14 +12,17 @@ import uuid
 import shutil
 import socket 
 import logging
+from multiprocessing import Pool
 
 from xtlib import utils
 from xtlib import errors
 from xtlib import pc_utils
-from xtlib.storage import mongo_v2
 from xtlib import constants
 from xtlib import file_utils
 from xtlib import run_helper
+from xtlib import store_utils
+from xtlib.storage import mongo_v2
+from xtlib.helpers import xt_config
 
 from xtlib.console import console
 from xtlib.helpers.bag import Bag
@@ -51,16 +54,19 @@ def store_from_context(context):
         mongo.update_job_stats = context.update_job_stats
         mongo.update_run_stats = context.update_run_stats
         mongo.add_log_records = context.add_log_records
+        mongo.buffer_metrics = context.buffer_metrics
 
     return store
 
-# public function
-def Store(storage_creds=None, config=None, max_retries=10, run_cache_dir=None, feedback_enabled=True,  mongo_conn_str=None, 
-        provider_code_path=None, validate_creds=False):
-    '''
-    This function peeks at the version number in the storage and chooses StoreV1 or StoreV2 as the store 
-    class to return an instance of.
-    '''
+def init_store_utils(use_v1):
+    if use_v1:
+        # JIT import to minimize impact of V1
+        from xtlib.storage_v1.store_v1 import build_store_utils_v1
+        build_store_utils_v1()
+    else:
+        build_store_utils_v2()
+
+def is_version1_store(config=None, storage_creds=None, provider_code_path=None):
     if config:
         if not storage_creds:
             storage_creds = config.get_storage_creds()
@@ -78,6 +84,21 @@ def Store(storage_creds=None, config=None, max_retries=10, run_cache_dir=None, f
     else:
         use_v1 = provider.does_container_exist(constants.INFO_CONTAINER_V1)
 
+    return use_v1, provider
+
+# public function
+def Store(storage_creds=None, config=None, max_retries=25, run_cache_dir=None, feedback_enabled=True,  mongo_conn_str=None, 
+        provider_code_path=None, validate_creds=False):
+    '''
+    This function peeks at the version number in the storage and chooses StoreV1 or StoreV2 as the store 
+    class to return an instance of.
+    '''
+    use_v1, provider = is_version1_store(config=config, storage_creds=storage_creds, 
+        provider_code_path=provider_code_path)
+
+    # initialize functions of data of share_utils to use v1 or v2 storage format
+    init_store_utils(use_v1)
+
     if use_v1:
         #----- V1 store ----
         # JIT import to minimize impact of V1
@@ -122,7 +143,7 @@ class StoreV2():
 
     '''
     
-    def __init__(self, storage_creds=None, config=None, max_retries=10, run_cache_dir=None, feedback_enabled=True,  mongo_conn_str=None, 
+    def __init__(self, storage_creds=None, config=None, max_retries=25, run_cache_dir=None, feedback_enabled=True,  mongo_conn_str=None, 
         provider_code_path=None, validate_creds=False):
         '''This is the constructor for the Store class. '''
 
@@ -156,11 +177,12 @@ class StoreV2():
                 errors.service_error("Azure Storage service credentials not set correctly" + 
                     "; use 'xt config' to correct")
 
-        self.mongo = mongo_v2.MongoDB2(mongo_conn_str, run_cache_dir=run_cache_dir) if mongo_conn_str else None
+        self.mongo = mongo_v2.MongoDB2(mongo_conn_str, run_cache_dir=run_cache_dir, store=self) if mongo_conn_str else None
         self.mongo_conn_str = mongo_conn_str
         self.run_cache_dir = run_cache_dir
         self.provider_code_path = provider_code_path
         self.storage_creds = storage_creds
+        self.format = constants.STORAGE_FORMAT_V2
 
         self.store_type = storage_creds["provider"]
 
@@ -170,6 +192,10 @@ class StoreV2():
         if self.mongo:
             self.helper.validate_storage_and_mongo(self.mongo)
 
+    def init_store_utils_if_needed(self):
+        ''' this function works by forcing creation of JIT store object, which calls init_store_utils()'''
+        pass
+
     def get_name(self):
         return self.helper.get_name()
         
@@ -246,9 +272,12 @@ class StoreV2():
         '''
 
         result = self.helper.delete_workspace(ws_name)
+        console.print("delete_workspace: result from storage provider: {}".format(result))
+
         if result:
             # remove associated mongo info
-            self.mongo.delete_workspace_if_needed(ws_name)
+            result2 = self.mongo.delete_workspace_if_needed(ws_name)
+            console.print("delete_workspace: result from mongo: {}".format(result2))
 
         return result
     
@@ -847,10 +876,10 @@ class StoreV2():
         return self.helper.get_job_names()
 
     def get_all_runs(self, aggregator_dest, ws_name, job_or_exper_name, filter_dict=None, fields_dict=None, use_cache=False, 
-        fn_cache=None, first_count=None, last_count=None, sort_dict=None, batch_size=None):
+        fn_cache=None, batch_size=None):
 
         return self.mongo.get_all_runs(aggregator_dest, ws_name, job_or_exper_name, filter_dict, fields_dict, use_cache,
-            fn_cache, first_count, last_count, sort_dict, batch_size=batch_size)
+            fn_cache, batch_size=batch_size)
 
     def wrapup_run(self, ws_name, run_name, aggregate_dest, dest_name, status, exit_code, primary_metric, maximize_metric, 
         report_rollup, rundir, after_files_list, log_events=True, capture_files=True, job_id=None, is_parent=False, 
@@ -948,6 +977,48 @@ class StoreV2():
         rf = self.run_files(ws_name, run_name, use_blobs=True)
         return rf.upload_files(run_folder, source_wildcard, exclude_dirs_and_files=exclude_dirs_and_files)
 
+    def read_log_records(self, ws_name, run_name):
+        text = self.read_run_file(ws_name, run_name, "run.log")
+        records = utils.load_json_records(text)
+        id = "{}/{}".format(ws_name, run_name)
+
+        # make this look like a mongo record so consuming code is happy
+        for record in records:
+            record["key"] = id
+            record["_id"] = str(uuid.uuid4())   # GUID
+
+        return records
+
+    def get_log_records_for_runs(self, ws_name, id_list):
+        # use worker processes to make this faster
+
+        # build workitems to interate over
+        work_items = []
+        config = xt_config.get_merged_config()    
+
+        for id in id_list:
+            run_name = id.split("/")[-1]
+            work_items.append( [config, ws_name, run_name] )
+
+        num_workers = 20
+        print("  reading log_records from storage ({} runs, {} worker processes): ".format(len(id_list), num_workers), 
+            end="")
+
+        pool = Pool(processes=num_workers) 
+
+        started = time.time()
+        results = pool.map(worker_read_log_records, work_items)
+        elapsed = time.time() - started
+
+        print(" {:.0f} secs elapsed".format(elapsed))
+
+        # flatten records into single list
+        all_run_records = []
+        for records in results:
+            all_run_records += records
+
+        return all_run_records
+
     def download_file_from_run(self, ws_name, run_name, run_fn, dest_fn, job_id=None):
         '''download the run file 'run_fn' to the local file 'dest_fn'.
         '''
@@ -1011,7 +1082,7 @@ class StoreV2():
         self.helper.write_job_info_file(ws_name, job_id, text)
 
         # update mongo-db for fast access
-        self.mongo.update_job(dd)
+        self.mongo.create_mongo_job(dd)
 
     def log_job_event(self, ws_name, job_id, event_name, data_dict=None, event_time=None):
         if not event_time:
@@ -1156,12 +1227,60 @@ class StoreV2():
 #   wp = store.WorkspaceFiles(ws_name="ws1")  
 #   wp.create_file("test.txt", "this is test.txt contents")
 
-#---- share helpers ----
+def build_store_utils_v2():
+    # update PROPERTIES of store_utils for version 2 values
+    store_utils.DATA_STORE_ROOT = make_share_name("data")
+    store_utils.MODELS_STORE_ROOT = make_share_name("models")
+    store_utils.STORAGE_FORMAT = constants.STORAGE_FORMAT_V2
+
+    # update FUNCTIONS of store_utils for version 1 behavior
+    store_utils.make_share_name = make_share_name
+    store_utils.is_share_name = is_share_name
+    store_utils.get_run_path = get_run_path
+    store_utils.get_jobs_container = get_jobs_container
+    store_utils.make_id = make_id
+        
+#---- store helpers (for replacement of store_utils) ----
 def make_share_name(name):
     return "00-share-{}".format(name)
 
 def is_share_name(name):
     return isinstance(name, str) and name.startswith("00-share-")
 
+def get_run_path(job_id, run_name):
+    path = "jobs/{}/runs/{}".format(job_id, run_name)
+    return path
 
+def get_jobs_container(ws_name):
+    return ws_name    
+
+def make_id(ws_name, obj_name):
+    if "/" in obj_name:
+        id = obj_name
+    else:
+        id = ws_name + "/" + obj_name
+
+    return id
+
+def simplify_id(record):
+    if "_id" in record:
+        _id = record["_id"]
+        if "/" in _id:
+            _id = _id.split("/")[-1]
+            record["_id"] = _id
+
+def simplify_records_id(records):
+    if records:
+        for r in records:
+            simplify_id(r)
+
+# inner function (runs on each background process)
+def worker_read_log_records(info):
+    config, ws_name, run_name = info
+    #print("reading log...", run_name, flush=True)
+
+    store = Store(config=config)
+    records = store.read_log_records(ws_name, run_name)
+    #print(".", end="", flush=True)
+    return records
 
