@@ -457,7 +457,7 @@ class MongoDB():
         console.diag("after get_all_experiments()")        
         return exper_names
 
-    def get_ws_runs(self, ws_name, filter_dict=None, include_log_records=False, first_count=None, last_count=None, sort_dict=None):
+    def get_ws_runs(self, ws_name, filter_dict=None, include_log_records=False):
         '''
         design issue: we cannot use a single cache file for different filter requests.  Possible fixes:
             - do not cache here (current option)
@@ -473,11 +473,10 @@ class MongoDB():
 
         fn_cache = fn_cache.replace("$aggregator", ws_name)
 
-        return self.get_all_runs(None, ws_name, None, filter_dict, fields_dict, use_cache=False, fn_cache=fn_cache, first_count=first_count, 
-            last_count=last_count, sort_dict=sort_dict)
+        return self.get_all_runs(None, ws_name, None, filter_dict, fields_dict, use_cache=False, fn_cache=fn_cache)
 
     def get_all_runs(self, aggregator_dest, ws_name, job_or_exper_name, filter_dict=None, fields_dict=None, use_cache=True, 
-        fn_cache=None, first_count=None, last_count=None, sort_dict=None, batch_size=None):
+        fn_cache=None, batch_size=None):
         '''
         cache design: 
             - organized all cached run information by the way it was accessed: a folder for each workspace (created on demand), 
@@ -527,7 +526,6 @@ class MongoDB():
         if target:
             filter_dict["end_id"] = {"$gt": target}
 
-        #console.print("  mongo: filter: {}, fields: {}, sort: {}".format(filter_dict, fields_dict, sort_dict))
         console.diag("  mongo: filter: {}, fields: {}".format(filter_dict, fields_dict))
 
         #records = self.mongo_db[ws_name].find(filter_dict, fields_dict)  
@@ -569,7 +567,7 @@ class MongoDB():
 
         return records
 
-    def update_runs_from_filter(self, ws_name, filter, dd, clear=False, upsert=True):
+    def update_run_stats_from_list(self, ws_name, id_list, dd, clear=False, upsert=True):
 
         if self.update_run_stats:
             if clear:
@@ -577,12 +575,31 @@ class MongoDB():
             else:
                 update_doc = { "$set": dd}
 
+            update_filter = {"_id": {"$in": id_list}}
+
             # update, create prop if needed
-            result = self.mongo_with_retries("update_runs_from_filter", lambda: self.mongo_db[ws_name].update_many( filter, update_doc, upsert=upsert) )
+            result = self.mongo_with_retries("update_run_stats_from_list", lambda: \
+                self.mongo_db[ws_name].update_many( update_filter, update_doc, upsert=upsert) )
+
             return result
 
     #---- JOBS ----
 
+    def update_job_stats_from_list(self, ws_name, id_list, dd, clear=False, upsert=True):
+
+        if self.update_job_stats:
+            if clear:
+                update_doc = { "$unset": dd}
+            else:
+                update_doc = { "$set": dd}
+
+            update_filter = {"_id": {"$in": id_list}}
+
+            # update, create prop if needed
+            result = self.mongo_with_retries("update_job_stats_from_list", \
+                lambda: self.mongo_db["__jobs__"].update_many( update_filter, update_doc, upsert=upsert) )
+            return result
+
     def does_jobs_exist(self, ws_name):
         # does document for ws_name exist?
         job_id = "job1"
@@ -703,7 +720,7 @@ class MongoDB():
 
         return records
 
-    def update_job(self, jd):
+    def create_mongo_job(self, jd):
         ws_name = jd["ws_name"]
         job_id = jd["job_id"]
 
@@ -860,4 +877,8 @@ class MongoDB():
         run_num = self.get_next_run_id(ws_name)
         run_name = "run" + str(run_num)
 
-        return run_name
\ No newline at end of file
+        return run_name
+
+    def on_run_close(self):
+        # nothing for v1 to do on this API call
+        pass
