@@ -72,22 +72,10 @@ class StoreV1():
 
     '''
     
-    def __init__(self, storage_creds=None, config=None, max_retries=10, run_cache_dir=None, feedback_enabled=True,  mongo_conn_str=None, 
+    def __init__(self, storage_creds=None, config=None, max_retries=25, run_cache_dir=None, feedback_enabled=True,  mongo_conn_str=None, 
         provider_code_path=None, validate_creds=False):
         '''This is the constructor for the Store class. '''
 
-        # update PROPERTIES of store_utils for version 1 values
-        store_utils.DATA_STORE_ROOT = make_share_name("data")
-        store_utils.MODELS_STORE_ROOT = make_share_name("models")
-        store_utils.STORAGE_FORMAT = constants.STORAGE_FORMAT_V1
-
-        # update FUNCTIONS of store_utils for version 1 behavior
-        store_utils.make_share_name = make_share_name
-        store_utils.is_share_name = is_share_name
-        store_utils.get_run_path = get_run_path
-        store_utils.get_jobs_container = get_jobs_container
-        store_utils.make_id = make_id
-
         self.feedback_enabled = feedback_enabled
 
         if not mongo_conn_str and config:
@@ -119,6 +107,7 @@ class StoreV1():
         self.run_cache_dir = run_cache_dir
         self.provider_code_path = provider_code_path
         self.storage_creds = storage_creds
+        self.format = constants.STORAGE_FORMAT_V1
 
         self.store_type = storage_creds["provider"]
 
@@ -127,6 +116,10 @@ class StoreV1():
 
         self.helper.validate_storage_and_mongo(self.mongo)
 
+    def init_store_utils_if_needed(self):
+        ''' this function works by forcing creation of JIT store object, which calls init_store_utils()'''
+        pass
+
     def get_name(self):
         return self.helper.get_name()
         
@@ -832,14 +825,14 @@ class StoreV1():
     def get_job_names(self, filter_dict=None, fields_dict=None):
         return self.mongo.get_job_names(filter_dict, fields_dict)
 
-    def get_ws_runs(self, ws_name, filter_dict=None, include_log_records=False, first_count=None, last_count=None, sort_dict=None):
-        return self.mongo.get_ws_runs(ws_name, filter_dict, include_log_records, first_count, last_count, sort_dict)
+    def get_ws_runs(self, ws_name, filter_dict=None, include_log_records=False):
+        return self.mongo.get_ws_runs(ws_name, filter_dict, include_log_records)
 
     def get_all_runs(self, aggregator_dest, ws_name, job_or_exper_name, filter_dict=None, fields_dict=None, use_cache=False, 
-        fn_cache=None, first_count=None, last_count=None, sort_dict=None, batch_size=None):
+        fn_cache=None, batch_size=None):
 
         return self.mongo.get_all_runs(aggregator_dest, ws_name, job_or_exper_name, filter_dict, fields_dict, use_cache,
-            fn_cache, first_count, last_count, sort_dict, batch_size=batch_size)
+            fn_cache, batch_size=batch_size)
 
     def wrapup_run(self, ws_name, run_name, aggregate_dest, dest_name, status, exit_code, primary_metric, maximize_metric, 
         report_rollup, rundir, after_files_list, log_events=True, capture_files=True, job_id=None, is_parent=False, 
@@ -1145,6 +1138,19 @@ class StoreV1():
 #   wp = store.WorkspaceFiles(ws_name="ws1")  
 #   wp.create_file("test.txt", "this is test.txt contents")
 
+def build_store_utils_v1():
+    # update PROPERTIES of store_utils for version 1 values
+    store_utils.DATA_STORE_ROOT = make_share_name("data")
+    store_utils.MODELS_STORE_ROOT = make_share_name("models")
+    store_utils.STORAGE_FORMAT = constants.STORAGE_FORMAT_V1
+
+    # update FUNCTIONS of store_utils for version 1 behavior
+    store_utils.make_share_name = make_share_name
+    store_utils.is_share_name = is_share_name
+    store_utils.get_run_path = get_run_path
+    store_utils.get_jobs_container = get_jobs_container
+    store_utils.make_id = make_id
+        
 #---- store helpers (for replacement of store_utils) ----
 def make_share_name(name):
     return "xts-{}-xts".format(name)
