@@ -4,33 +4,28 @@
 #
 # store_utils.py: store-related helper functions
 '''
-NOTE: when we are running with version 1 storage, we dynamically 
-REPLACE ALL OF THE BELOW with their version 1 equivalents 
+NOTE: when store.init_store_utils() is called, we dynamically 
+REPLACE ALL OF THE BELOW with their version 1 or version 2 equivalents 
 (functions and properties).
 '''
 from xtlib import constants
 
 def make_share_name(name):
-    return "00-share-{}".format(name)
+   raise Exception("store-utils not yet initialized")
 
 def is_share_name(name):
-    return isinstance(name, str) and name.startswith("00-share-")
+   raise Exception("store-utils not yet initialized")
 
 def get_run_path(job_id, run_name):
-    path = "jobs/{}/runs/{}".format(job_id, run_name)
-    return path
+   raise Exception("store-utils not yet initialized")
 
 def get_jobs_container(ws_name):
-    return ws_name    
+   raise Exception("store-utils not yet initialized")
 
 def make_id(ws_name, obj_name):
-    if "/" in obj_name:
-        id = obj_name
-    else:
-        id = ws_name + "/" + obj_name
-
-    return id
+   raise Exception("store-utils not yet initialized")
 
+# version independent function
 def simplify_id(record):
     if "_id" in record:
         _id = record["_id"]
@@ -38,12 +33,13 @@ def simplify_id(record):
             _id = _id.split("/")[-1]
             record["_id"] = _id
 
+# version independent function
 def simplify_records_id(records):
     if records:
         for r in records:
             simplify_id(r)
 
-MODELS_STORE_ROOT = make_share_name("models")
-DATA_STORE_ROOT = make_share_name("data")
-STORAGE_FORMAT = constants.STORAGE_FORMAT
+MODELS_STORE_ROOT = None
+DATA_STORE_ROOT = None
+STORAGE_FORMAT = None
 
